from _typeshed import Incomplete

from ..violation import Violation
from .base import BaseFormatter

COLORS: dict[str, str]
COLORS_OFF: dict[str, str]

class SimpleFormatter(BaseFormatter):
    error_format: str
    def format(self, error: Violation) -> str | None: ...

class Default(SimpleFormatter):
    error_format: str
    def after_init(self) -> None: ...

class Pylint(SimpleFormatter):
    error_format: str

class FilenameOnly(SimpleFormatter):
    error_format: str
    filenames_already_printed: Incomplete
    def after_init(self) -> None: ...
    def show_source(self, error: Violation) -> str | None: ...
    def format(self, error: Violation) -> str | None: ...

class Nothing(BaseFormatter):
    def format(self, error: Violation) -> str | None: ...
    def show_source(self, error: Violation) -> str | None: ...



from typing import Final

from pyasn1.type.constraint import ConstraintsIntersection, ValueRangeConstraint
from pyasn1.type.namedtype import NamedTypes
from pyasn1.type.univ import Integer, OctetString, Sequence

MAXINT: Final[Integer]
rangeInt0ToMaxConstraint: ValueRangeConstraint

class Integer0ToMax(Integer):
    subtypeSpec: ConstraintsIntersection

class Size(Integer0ToMax): ...
class Cookie(OctetString): ...

class RealSearchControlValue(Sequence):
    componentType: NamedTypes

def paged_search_control(criticality: bool = False, size: int = 10, cookie=None): ...



from __future__ import annotations

import _threading_local
import threading

loc = threading.local()
loc.foo = 42
del loc.foo
loc.baz = ["spam", "eggs"]
del loc.baz

l2 = _threading_local.local()
l2.asdfasdf = 56
del l2.asdfasdf



from __future__ import annotations

from collections.abc import Iterable

import requests

# =================================================================================================
# Regression test for #7988 (multiple files should be allowed for the "files" argument)
# This snippet comes from the requests documentation
# (https://requests.readthedocs.io/en/latest/user/advanced/#post-multiple-multipart-encoded-files),
# so should pass a type checker without error
# =================================================================================================


url = "https://httpbin.org/post"
multiple_files = [
    ("images", ("foo.png", open("foo.png", "rb"), "image/png")),
    ("images", ("bar.png", open("bar.png", "rb"), "image/png")),
]
r = requests.post(url, files=multiple_files)


# =================================================================================
# Tests for various different types being passed into the "data" parameter
# (These all return "Any", so there's not much value in using assert_type here.)
# (Just test that type checkers don't emit an error if it doesn't fail at runtime.)
# =================================================================================


# Arbitrary iterable
def gen() -> Iterable[bytes]:
    yield b"foo"
    yield b"bar"


requests.post("http://httpbin.org/anything", data=gen()).json()["data"]

# bytes
requests.post("http://httpbin.org/anything", data=b"foobar").json()["data"]

# str
requests.post("http://httpbin.org/anything", data="foobar").json()["data"]

# Files
requests.post("http://httpbin.org/anything", data=open("/tmp/foobar", "rb", encoding="UTF-8")).json()["data"]
requests.post("http://httpbin.org/anything", data=open("/tmp/foobar", "r", encoding="UTF-8")).json()["data"]

# Mappings
requests.post("http://httpbin.org/anything", data={b"foo": b"bar"}).json()["form"]
requests.post("http://httpbin.org/anything", data={"foo": "bar"}).json()["form"]

# mappings represented by an list/tuple of key-values pairs
requests.post("http://httpbin.org/anything", data=[(b"foo", b"bar")]).json()["form"]
requests.post("http://httpbin.org/anything", data=[("foo", "bar")]).json()["form"]
requests.post("http://httpbin.org/anything", data=((b"foo", b"bar"),)).json()["form"]
requests.post("http://httpbin.org/anything", data=(("foo", "bar"),)).json()["form"]



from _typeshed import Incomplete, SupportsGetItem
from collections.abc import Callable, Collection, Hashable, Iterable, Mapping, Sequence
from typing import Any, Literal, Protocol, SupportsIndex, overload, type_check_only
from typing_extensions import Self, TypeAlias, deprecated

import numpy as np
import pandas as pd
from numpy.random import BitGenerator, Generator as RandomGenerator, SeedSequence
from numpy.typing import ArrayLike, NDArray
from pandas._typing import ListLikeU
from pandas.core.base import IndexOpsMixin
from pyproj import CRS
from shapely import Geometry, MultiPolygon, Point, Polygon
from shapely.geometry.base import BaseGeometry

from .array import GeometryArray, _Array1D
from .geodataframe import GeoDataFrame
from .geoseries import GeoSeries
from .sindex import SpatialIndex

@type_check_only
class _SupportsToWkt(Protocol):
    def to_wkt(self) -> str: ...

@type_check_only
class _SupportsGeoInterface(Protocol):  # noqa: Y046
    @property
    def __geo_interface__(self) -> dict[str, Any]: ...  # values are arbitrary

_ConvertibleToCRS: TypeAlias = str | int | tuple[str, str] | list[str] | dict[str, Incomplete] | _SupportsToWkt
_AffinityOrigin: TypeAlias = Literal["center", "centroid"] | Point | tuple[float, float] | tuple[float, float, float]
_ClipMask: TypeAlias = GeoDataFrame | GeoSeries | Polygon | MultiPolygon | tuple[float, float, float, float]  # noqa: Y047
# np.floating[Any] because precision is not important
_BboxLike: TypeAlias = Sequence[float] | NDArray[np.floating[Any]] | Geometry | GeoDataFrame | GeoSeries  # noqa: Y047
_MaskLike: TypeAlias = dict[str, Incomplete] | Geometry | GeoDataFrame | GeoSeries  # noqa: Y047

# Cannot use IndexOpsMixin[Geometry] because of IndexOpsMixin type variable bounds
_GeoListLike: TypeAlias = ArrayLike | Sequence[Geometry] | IndexOpsMixin[Any]
_ConvertibleToGeoSeries: TypeAlias = Geometry | Mapping[int, Geometry] | Mapping[str, Geometry] | _GeoListLike  # noqa: Y047

# Cannot use pd.Series[Geometry] because of pd.Series type variable bounds
_GeomSeq: TypeAlias = Sequence[Geometry] | NDArray[np.object_] | pd.Series[Any] | GeometryArray | GeoSeries
_GeomCol: TypeAlias = Hashable | _GeomSeq  # name of column or column values  # noqa: Y047
# dict[Any, Any] because of variance issues
_ConvertibleToDataFrame: TypeAlias = (  # noqa: Y047
    ListLikeU | pd.DataFrame | dict[Any, Any] | Iterable[ListLikeU | tuple[Hashable, ListLikeU] | dict[Any, Any]]
)

def is_geometry_type(data: object) -> bool: ...

class GeoPandasBase:
    @property
    def area(self) -> pd.Series[float]: ...
    @property
    def crs(self) -> CRS | None: ...
    @crs.setter
    def crs(self, value: _ConvertibleToCRS | None) -> None: ...
    @property
    def geom_type(self) -> pd.Series[str]: ...
    @property
    def type(self) -> pd.Series[str]: ...
    @property
    def length(self) -> pd.Series[float]: ...
    @property
    def is_valid(self) -> pd.Series[bool]: ...
    def is_valid_reason(self) -> pd.Series[str]: ...
    def is_valid_coverage(self, *, gap_width: float = 0.0) -> bool: ...
    def invalid_coverage_edges(self, *, gap_width: float = 0.0) -> GeoSeries: ...
    @property
    def is_empty(self) -> pd.Series[bool]: ...
    def count_coordinates(self) -> pd.Series[int]: ...
    def count_geometries(self) -> pd.Series[int]: ...
    def count_interior_rings(self) -> pd.Series[int]: ...
    @property
    def is_simple(self) -> pd.Series[bool]: ...
    @property
    def is_ring(self) -> pd.Series[bool]: ...
    @property
    def is_ccw(self) -> pd.Series[bool]: ...
    @property
    def is_closed(self) -> pd.Series[bool]: ...
    @property
    def has_z(self) -> pd.Series[bool]: ...
    @property
    def has_m(self) -> pd.Series[bool]: ...
    def get_precision(self) -> pd.Series[float]: ...
    def get_geometry(self, index: SupportsIndex | ArrayLike) -> GeoSeries: ...
    @property
    def boundary(self) -> GeoSeries: ...
    @property
    def centroid(self) -> GeoSeries: ...
    def concave_hull(self, ratio: float = 0.0, allow_holes: bool = False) -> GeoSeries: ...
    def constrained_delaunay_triangles(self) -> GeoSeries: ...
    @property
    def convex_hull(self) -> GeoSeries: ...
    def delaunay_triangles(self, tolerance: float | ArrayLike = 0.0, only_edges: bool | ArrayLike = False) -> GeoSeries: ...
    def voronoi_polygons(
        self, tolerance: float | ArrayLike = 0.0, extend_to: Geometry | None = None, only_edges: bool = False
    ) -> GeoSeries: ...
    @property
    def envelope(self) -> GeoSeries: ...
    def minimum_rotated_rectangle(self) -> GeoSeries: ...
    @property
    def exterior(self) -> GeoSeries: ...
    def extract_unique_points(self) -> GeoSeries: ...
    def offset_curve(
        self,
        distance: float | ArrayLike,
        quad_segs: int = 8,
        join_style: Literal["round", "bevel", "mitre"] = "round",
        mitre_limit: float = 5.0,
    ) -> GeoSeries: ...
    @property
    def interiors(self) -> pd.Series[Any]: ...  # Cannot use pd.Series[BaseGeometry]
    def remove_repeated_points(self, tolerance: float = 0.0) -> GeoSeries: ...
    def set_precision(
        self, grid_size: float, mode: Literal["valid_output", "pointwise", "keep_collapsed"] = "valid_output"
    ) -> GeoSeries: ...
    def representative_point(self) -> GeoSeries: ...
    def minimum_bounding_circle(self) -> GeoSeries: ...
    def maximum_inscribed_circle(self, *, tolerance: float | ArrayLike | None = None) -> GeoSeries: ...
    def minimum_bounding_radius(self) -> pd.Series[float]: ...
    def minimum_clearance(self) -> pd.Series[float]: ...
    def minimum_clearance_line(self) -> GeoSeries: ...
    def normalize(self) -> GeoSeries: ...
    def orient_polygons(self, *, exterior_cw: bool = False) -> GeoSeries: ...
    def make_valid(self, *, method: Literal["linework", "structure"] = "linework", keep_collapsed: bool = True) -> GeoSeries: ...
    def reverse(self) -> GeoSeries: ...
    def segmentize(self, max_segment_length: float | ArrayLike) -> GeoSeries: ...
    def transform(
        self, transformation: Callable[[NDArray[np.float64]], NDArray[np.float64]], include_z: bool = False
    ) -> GeoSeries: ...
    def force_2d(self) -> GeoSeries: ...
    def force_3d(self, z: float | ArrayLike = 0) -> GeoSeries: ...
    def line_merge(self, directed: bool = False) -> GeoSeries: ...
    @property
    @deprecated("Use method `union_all` instead.")
    def unary_union(self) -> BaseGeometry: ...
    def union_all(
        self, method: Literal["coverage", "unary", "disjoint_subset"] = "unary", *, grid_size: float | None = None
    ) -> BaseGeometry: ...
    def intersection_all(self) -> BaseGeometry: ...
    def contains(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[bool]: ...
    def contains_properly(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[bool]: ...
    def dwithin(self, other: GeoSeries | Geometry, distance: float | ArrayLike, align: bool | None = None) -> pd.Series[bool]: ...
    def geom_equals(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[bool]: ...
    def geom_equals_exact(
        self, other: GeoSeries | Geometry, tolerance: float | ArrayLike, align: bool | None = None
    ) -> pd.Series[bool]: ...
    def geom_equals_identical(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[bool]: ...
    def crosses(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[bool]: ...
    def disjoint(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[bool]: ...
    def intersects(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[bool]: ...
    def overlaps(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[bool]: ...
    def touches(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[bool]: ...
    def within(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[bool]: ...
    def covers(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[bool]: ...
    def covered_by(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[bool]: ...
    def distance(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[float]: ...
    def hausdorff_distance(
        self, other: GeoSeries | Geometry, align: bool | None = None, densify: float | ArrayLike | None = None
    ) -> pd.Series[float]: ...
    def frechet_distance(
        self, other: GeoSeries | Geometry, align: bool | None = None, densify: float | ArrayLike | None = None
    ) -> pd.Series[float]: ...
    def difference(self, other: GeoSeries | Geometry, align: bool | None = None) -> GeoSeries: ...
    def symmetric_difference(self, other: GeoSeries | Geometry, align: bool | None = None) -> GeoSeries: ...
    def union(self, other: GeoSeries | Geometry, align: bool | None = None) -> GeoSeries: ...
    def intersection(self, other: GeoSeries | Geometry, align: bool | None = None) -> GeoSeries: ...
    def clip_by_rect(self, xmin: float, ymin: float, xmax: float, ymax: float) -> GeoSeries: ...
    def shortest_line(self, other: GeoSeries | Geometry, align: bool | None = None) -> GeoSeries: ...
    def snap(self, other: GeoSeries | Geometry, tolerance: float | ArrayLike, align: bool | None = None) -> GeoSeries: ...
    def shared_paths(self, other: GeoSeries | Geometry, align: bool | None = None): ...
    @property
    def bounds(self) -> pd.DataFrame: ...
    @property
    def total_bounds(self) -> _Array1D[np.float64]: ...
    @property
    def sindex(self) -> SpatialIndex: ...
    @property
    def has_sindex(self) -> bool: ...
    def buffer(
        self,
        distance: float | ArrayLike,
        resolution: int = 16,
        cap_style: Literal["round", "square", "flat"] = "round",
        join_style: Literal["round", "mitre", "bevel"] = "round",
        mitre_limit: float = 5.0,
        single_sided: bool = False,
        **kwargs,
    ) -> GeoSeries: ...
    def simplify(self, tolerance: float | ArrayLike, preserve_topology: bool = True) -> GeoSeries: ...
    def simplify_coverage(self, tolerance: float | ArrayLike, *, simplify_boundary: bool = True) -> GeoSeries: ...
    def relate(self, other: GeoSeries | Geometry, align: bool | None = None) -> pd.Series[str]: ...
    def relate_pattern(self, other: GeoSeries | Geometry, pattern: str, align: bool | None = None) -> pd.Series[bool]: ...
    def project(self, other: GeoSeries | Geometry, normalized: bool = False, align: bool | None = None) -> pd.Series[float]: ...
    def interpolate(self, distance: float | ArrayLike, normalized: bool = False) -> GeoSeries: ...
    def affine_transform(self, matrix: Collection[float]) -> GeoSeries: ...
    def translate(self, xoff: float = 0.0, yoff: float = 0.0, zoff: float = 0.0) -> GeoSeries: ...
    def rotate(self, angle: float, origin: _AffinityOrigin = "center", use_radians: bool = False) -> GeoSeries: ...
    def scale(
        self, xfact: float = 1.0, yfact: float = 1.0, zfact: float = 1.0, origin: _AffinityOrigin = "center"
    ) -> GeoSeries: ...
    def skew(
        self, xs: float = 0.0, ys: float = 0.0, origin: _AffinityOrigin = "center", use_radians: bool = False
    ) -> GeoSeries: ...
    @property
    def cx(self) -> SupportsGetItem[tuple[SupportsIndex | slice, SupportsIndex | slice], Self]: ...
    def get_coordinates(
        self, include_z: bool = False, ignore_index: bool = False, index_parts: bool = False, *, include_m: bool = False
    ) -> pd.DataFrame: ...
    def hilbert_distance(
        self, total_bounds: tuple[float, float, float, float] | Iterable[float] | None = None, level: int = 16
    ) -> pd.Series[int]: ...
    @overload
    def sample_points(
        self,
        size: int | ArrayLike,
        method: str = "uniform",
        seed: None = None,
        rng: int | ArrayLike | SeedSequence | BitGenerator | RandomGenerator | None = None,
        **kwargs,
    ) -> GeoSeries: ...
    @overload
    @deprecated("Parameter `seed` is deprecated. Use `rng` instead.")
    def sample_points(
        self,
        size: int | ArrayLike,
        method: str = "uniform",
        *,
        seed: int | ArrayLike | SeedSequence | BitGenerator | RandomGenerator,
        rng: int | ArrayLike | SeedSequence | BitGenerator | RandomGenerator | None = None,
        **kwargs,
    ) -> GeoSeries: ...
    def build_area(self, node: bool = True) -> GeoSeries: ...
    @overload
    def polygonize(self, node: bool = True, full: Literal[False] = False) -> GeoSeries: ...
    @overload
    def polygonize(self, node: bool = True, *, full: Literal[True]) -> tuple[GeoSeries, GeoSeries, GeoSeries, GeoSeries]: ...
    @overload
    def polygonize(self, node: bool, full: Literal[True]) -> tuple[GeoSeries, GeoSeries, GeoSeries, GeoSeries]: ...



from logging import Logger
from typing import ClassVar, Final

from authlib.oauth2.rfc6749 import BaseGrant, TokenEndpointMixin

log: Logger
JWT_BEARER_GRANT_TYPE: Final[str]

class JWTBearerGrant(BaseGrant, TokenEndpointMixin):
    GRANT_TYPE = JWT_BEARER_GRANT_TYPE
    CLAIMS_OPTIONS: ClassVar[dict[str, dict[str, bool]]]
    LEEWAY: ClassVar[int]
    @staticmethod
    def sign(key, issuer, audience, subject=None, issued_at=None, expires_at=None, claims=None, **kwargs): ...
    def process_assertion_claims(self, assertion): ...
    def resolve_public_key(self, headers, payload): ...
    def validate_token_request(self) -> None: ...
    def create_token_response(self): ...
    def resolve_issuer_client(self, issuer) -> None: ...
    def resolve_client_key(self, client, headers, payload) -> None: ...
    def authenticate_user(self, subject) -> None: ...
    def has_granted_permission(self, client, user) -> None: ...



from argparse import ArgumentParser, Namespace

from gunicorn.app.base import Application

from .._types import _WSGIAppType

class WSGIApplication(Application):
    app_uri: str | None

    def init(self, parser: ArgumentParser, opts: Namespace, args: list[str]) -> None: ...
    def load_config(self) -> None: ...
    def load_wsgiapp(self) -> _WSGIAppType: ...
    def load_pasteapp(self) -> _WSGIAppType: ...
    def load(self) -> _WSGIAppType: ...

def run(prog: str | None = None) -> None: ...



import ctypes
import sys
from _typeshed import Incomplete
from typing import Literal
from typing_extensions import TypeAlias

if sys.platform == "win32":
    win32con_WM_COPYDATA: int
    def can_talk_to_agent() -> bool: ...

    ULONG_PTR: TypeAlias = ctypes.c_uint64 | ctypes.c_uint32

    class COPYDATASTRUCT(ctypes.Structure):
        num_data: Incomplete
        data_size: Incomplete
        data_loc: Incomplete

    class PageantConnection:
        def __init__(self) -> None: ...
        def send(self, data: bytes) -> None: ...
        def recv(self, n: int) -> Literal[""] | bytes: ...
        def close(self) -> None: ...



from antlr4.BufferedTokenStream import BufferedTokenStream as BufferedTokenStream
from antlr4.Lexer import Lexer as Lexer
from antlr4.Token import Token as Token

class CommonTokenStream(BufferedTokenStream):
    __slots__ = "channel"
    channel: int
    def __init__(self, lexer: Lexer, channel: int = 0) -> None: ...
    def adjustSeekIndex(self, i: int) -> int: ...
    def LB(self, k: int) -> Token | None: ...
    def LT(self, k: int) -> Token | None: ...
    def getNumberOfOnChannelTokens(self) -> int: ...



from _typeshed import Incomplete
from collections.abc import Callable

from networkx.algorithms.flow import edmonds_karp
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["k_components"]
default_flow_func = edmonds_karp

@_dispatchable
def k_components(G: Graph[_Node], flow_func: Callable[..., Incomplete] | None = None) -> dict[Incomplete, Incomplete]: ...
def build_k_number_dict(kcomps) -> dict[Incomplete, Incomplete]: ...



from .actions import Actions
from .attack_protection import AttackProtection
from .auth0 import Auth0
from .blacklists import Blacklists
from .branding import Branding
from .client_credentials import ClientCredentials
from .client_grants import ClientGrants
from .clients import Clients
from .connections import Connections
from .custom_domains import CustomDomains
from .device_credentials import DeviceCredentials
from .email_templates import EmailTemplates
from .emails import Emails
from .grants import Grants
from .guardian import Guardian
from .hooks import Hooks
from .jobs import Jobs
from .log_streams import LogStreams
from .logs import Logs
from .organizations import Organizations
from .resource_servers import ResourceServers
from .roles import Roles
from .rules import Rules
from .rules_configs import RulesConfigs
from .self_service_profiles import SelfServiceProfiles
from .stats import Stats
from .tenants import Tenants
from .tickets import Tickets
from .user_blocks import UserBlocks
from .users import Users
from .users_by_email import UsersByEmail

__all__ = (
    "Auth0",
    "Actions",
    "AttackProtection",
    "Blacklists",
    "Branding",
    "ClientCredentials",
    "ClientGrants",
    "Clients",
    "Connections",
    "CustomDomains",
    "DeviceCredentials",
    "EmailTemplates",
    "Emails",
    "Grants",
    "Guardian",
    "Hooks",
    "Jobs",
    "LogStreams",
    "Logs",
    "Organizations",
    "ResourceServers",
    "Roles",
    "RulesConfigs",
    "Rules",
    "SelfServiceProfiles",
    "Stats",
    "Tenants",
    "Tickets",
    "UserBlocks",
    "UsersByEmail",
    "Users",
)



from collections.abc import Sequence
from typing import Any, Literal, SupportsIndex, overload

from ._enum import ParamEnum
from ._typing import ArrayLike, ArrayLikeSeq, GeoArray, OptGeoArrayLike, OptGeoArrayLikeSeq, OptGeoT
from .geometry import GeometryCollection, LineString, MultiLineString, MultiPoint, MultiPolygon, Point, Polygon
from .geometry.base import BaseGeometry, BaseMultipartGeometry
from .lib import Geometry

__all__ = [
    "BufferCapStyle",
    "BufferJoinStyle",
    "boundary",
    "buffer",
    "build_area",
    "centroid",
    "clip_by_rect",
    "concave_hull",
    "constrained_delaunay_triangles",
    "convex_hull",
    "delaunay_triangles",
    "envelope",
    "extract_unique_points",
    "make_valid",
    "maximum_inscribed_circle",
    "minimum_bounding_circle",
    "minimum_clearance_line",
    "minimum_rotated_rectangle",
    "node",
    "normalize",
    "offset_curve",
    "orient_polygons",
    "oriented_envelope",
    "point_on_surface",
    "polygonize",
    "polygonize_full",
    "remove_repeated_points",
    "reverse",
    "segmentize",
    "simplify",
    "snap",
    "voronoi_polygons",
]

class BufferCapStyle(ParamEnum):
    round = 1
    flat = 2
    square = 3

class BufferJoinStyle(ParamEnum):
    round = 1
    mitre = 2
    bevel = 3

@overload
def boundary(geometry: Point | MultiPoint, **kwargs) -> GeometryCollection: ...
@overload
def boundary(geometry: LineString | MultiLineString, **kwargs) -> MultiPoint: ...
@overload
def boundary(geometry: Polygon | MultiPolygon, **kwargs) -> MultiLineString: ...
@overload
def boundary(geometry: GeometryCollection | None, **kwargs) -> None: ...
@overload
def boundary(geometry: Geometry, **kwargs) -> BaseMultipartGeometry | Any: ...
@overload
def boundary(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def buffer(
    geometry: Geometry,
    distance: float,
    quad_segs: int = 8,
    cap_style: BufferJoinStyle | Literal["round", "square", "flat"] = "round",
    join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = "round",
    mitre_limit: float = 5.0,
    single_sided: bool = False,
    **kwargs,
) -> Polygon: ...
@overload
def buffer(
    geometry: None,
    distance: float,
    quad_segs: int = 8,
    cap_style: BufferJoinStyle | Literal["round", "square", "flat"] = "round",
    join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = "round",
    mitre_limit: float = 5.0,
    single_sided: bool = False,
    **kwargs,
) -> None: ...
@overload
def buffer(
    geometry: Geometry | None,
    distance: float,
    quad_segs: int = 8,
    cap_style: BufferJoinStyle | Literal["round", "square", "flat"] = "round",
    join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = "round",
    mitre_limit: float = 5.0,
    single_sided: bool = False,
    **kwargs,
) -> Polygon | None: ...
@overload
def buffer(
    geometry: OptGeoArrayLike,
    distance: ArrayLikeSeq[float],
    quad_segs: int = 8,
    cap_style: BufferJoinStyle | Literal["round", "square", "flat"] = "round",
    join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = "round",
    mitre_limit: float = 5.0,
    single_sided: bool = False,
    **kwargs,
) -> GeoArray: ...
@overload
def buffer(
    geometry: OptGeoArrayLikeSeq,
    distance: ArrayLike[float],
    quad_segs: int = 8,
    cap_style: BufferJoinStyle | Literal["round", "square", "flat"] = "round",
    join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = "round",
    mitre_limit: float = 5.0,
    single_sided: bool = False,
    **kwargs,
) -> GeoArray: ...
@overload
def offset_curve(
    geometry: Geometry,
    distance: float,
    quad_segs: SupportsIndex = 8,
    join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = "round",
    mitre_limit: float = 5.0,
    **kwargs,
) -> LineString | MultiLineString: ...
@overload
def offset_curve(
    geometry: None,
    distance: float,
    quad_segs: SupportsIndex = 8,
    join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = "round",
    mitre_limit: float = 5.0,
    **kwargs,
) -> None: ...
@overload
def offset_curve(
    geometry: Geometry | None,
    distance: float,
    quad_segs: SupportsIndex = 8,
    join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = "round",
    mitre_limit: float = 5.0,
    **kwargs,
) -> LineString | MultiLineString | None: ...
@overload
def offset_curve(
    geometry: OptGeoArrayLike,
    distance: ArrayLikeSeq[float],
    quad_segs: SupportsIndex = 8,
    join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = "round",
    mitre_limit: float = 5.0,
    **kwargs,
) -> GeoArray: ...
@overload
def offset_curve(
    geometry: OptGeoArrayLikeSeq,
    distance: ArrayLike[float],
    quad_segs: SupportsIndex = 8,
    join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = "round",
    mitre_limit: float = 5.0,
    **kwargs,
) -> GeoArray: ...
@overload
def centroid(geometry: Geometry, **kwargs) -> Point: ...
@overload
def centroid(geometry: None, **kwargs) -> None: ...
@overload
def centroid(geometry: Geometry | None, **kwargs) -> Point | None: ...
@overload
def centroid(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def clip_by_rect(geometry: Geometry, xmin: float, ymin: float, xmax: float, ymax: float, **kwargs) -> BaseGeometry: ...
@overload
def clip_by_rect(geometry: None, xmin: float, ymin: float, xmax: float, ymax: float, **kwargs) -> None: ...
@overload
def clip_by_rect(
    geometry: Geometry | None, xmin: float, ymin: float, xmax: float, ymax: float, **kwargs
) -> BaseGeometry | None: ...
@overload
def clip_by_rect(geometry: OptGeoArrayLikeSeq, xmin: float, ymin: float, xmax: float, ymax: float, **kwargs) -> GeoArray: ...
@overload
def concave_hull(geometry: Geometry, ratio: float = 0.0, allow_holes: bool = False, **kwargs) -> BaseGeometry: ...
@overload
def concave_hull(geometry: None, ratio: float = 0.0, allow_holes: bool = False, **kwargs) -> None: ...
@overload
def concave_hull(geometry: Geometry | None, ratio: float = 0.0, allow_holes: bool = False, **kwargs) -> BaseGeometry | None: ...
@overload
def concave_hull(geometry: OptGeoArrayLikeSeq, ratio: float = 0.0, allow_holes: bool = False, **kwargs) -> GeoArray: ...
@overload
def convex_hull(geometry: Geometry, **kwargs) -> BaseGeometry: ...
@overload
def convex_hull(geometry: None, **kwargs) -> None: ...
@overload
def convex_hull(geometry: Geometry | None, **kwargs) -> BaseGeometry | None: ...
@overload
def convex_hull(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def delaunay_triangles(
    geometry: Geometry, tolerance: float = 0.0, only_edges: Literal[False] = False, **kwargs
) -> GeometryCollection: ...
@overload
def delaunay_triangles(geometry: Geometry, tolerance: float, only_edges: Literal[True], **kwargs) -> MultiLineString: ...
@overload
def delaunay_triangles(geometry: Geometry, tolerance: float = 0.0, *, only_edges: Literal[True], **kwargs) -> MultiLineString: ...
@overload
def delaunay_triangles(
    geometry: Geometry, tolerance: float = 0.0, only_edges: bool = False, **kwargs
) -> GeometryCollection | MultiLineString: ...
@overload
def delaunay_triangles(geometry: None, tolerance: float = 0.0, only_edges: bool = False, **kwargs) -> None: ...
@overload
def delaunay_triangles(
    geometry: Geometry | None, tolerance: float = 0.0, only_edges: bool = False, **kwargs
) -> GeometryCollection | MultiLineString | None: ...
@overload
def delaunay_triangles(
    geometry: OptGeoArrayLike, tolerance: ArrayLike[float], only_edges: ArrayLikeSeq[bool], **kwargs
) -> GeoArray: ...
@overload
def delaunay_triangles(
    geometry: OptGeoArrayLike, tolerance: ArrayLike[float] = 0.0, *, only_edges: ArrayLikeSeq[bool], **kwargs
) -> GeoArray: ...
@overload
def delaunay_triangles(
    geometry: OptGeoArrayLike, tolerance: ArrayLikeSeq[float], only_edges: ArrayLike[bool] = False, **kwargs
) -> GeoArray: ...
@overload
def delaunay_triangles(
    geometry: OptGeoArrayLikeSeq, tolerance: ArrayLike[float] = 0.0, only_edges: ArrayLike[bool] = False, **kwargs
) -> GeoArray: ...
@overload
def constrained_delaunay_triangles(geometry: Geometry, **kwargs) -> GeometryCollection: ...
@overload
def constrained_delaunay_triangles(geometry: None, **kwargs) -> None: ...
@overload
def constrained_delaunay_triangles(geometry: Geometry | None, **kwargs) -> GeometryCollection | None: ...
@overload
def constrained_delaunay_triangles(geometry: OptGeoArrayLikeSeq | OptGeoArrayLike, **kwargs) -> GeoArray: ...
@overload
def envelope(geometry: Point, **kwargs) -> Point: ...
@overload
def envelope(geometry: Geometry, **kwargs) -> BaseGeometry: ...
@overload
def envelope(geometry: None, **kwargs) -> None: ...
@overload
def envelope(geometry: Geometry | None, **kwargs) -> BaseGeometry | None: ...
@overload
def envelope(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def extract_unique_points(geometry: Geometry, **kwargs) -> MultiPoint: ...
@overload
def extract_unique_points(geometry: None, **kwargs) -> None: ...
@overload
def extract_unique_points(geometry: Geometry | None, **kwargs) -> MultiPoint | None: ...
@overload
def extract_unique_points(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def build_area(geometry: Geometry, **kwargs) -> BaseGeometry: ...
@overload
def build_area(geometry: None, **kwargs) -> None: ...
@overload
def build_area(geometry: Geometry | None, **kwargs) -> BaseGeometry | None: ...
@overload
def build_area(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...

# make_valid with `method="linework"` only accepts `keep_collapsed=True`
@overload
def make_valid(
    geometry: Geometry, *, method: Literal["linework"] = "linework", keep_collapsed: Literal[True] = True, **kwargs
) -> BaseGeometry: ...
@overload
def make_valid(
    geometry: None, *, method: Literal["linework"] = "linework", keep_collapsed: Literal[True] = True, **kwargs
) -> None: ...
@overload
def make_valid(
    geometry: Geometry | None, *, method: Literal["linework"] = "linework", keep_collapsed: Literal[True] = True, **kwargs
) -> BaseGeometry | None: ...
@overload
def make_valid(
    geometry: OptGeoArrayLikeSeq, *, method: Literal["linework"] = "linework", keep_collapsed: Literal[True] = True, **kwargs
) -> GeoArray: ...
@overload
def make_valid(geometry: Geometry, *, method: Literal["structure"], keep_collapsed: bool = True, **kwargs) -> BaseGeometry: ...
@overload
def make_valid(geometry: None, *, method: Literal["structure"], keep_collapsed: bool = True, **kwargs) -> None: ...
@overload
def make_valid(
    geometry: Geometry | None, *, method: Literal["structure"], keep_collapsed: bool = True, **kwargs
) -> BaseGeometry | None: ...
@overload
def make_valid(
    geometry: OptGeoArrayLikeSeq, *, method: Literal["structure"], keep_collapsed: bool = True, **kwargs
) -> GeoArray: ...
@overload
def minimum_clearance_line(geometry: Point, **kwargs) -> Point: ...
@overload
def minimum_clearance_line(geometry: LineString | Polygon | BaseMultipartGeometry, **kwargs) -> Polygon: ...
@overload
def minimum_clearance_line(geometry: Geometry, **kwargs) -> Polygon | Point: ...
@overload
def minimum_clearance_line(geometry: None, **kwargs) -> None: ...
@overload
def minimum_clearance_line(geometry: Geometry | None, **kwargs) -> Polygon | Point | None: ...
@overload
def minimum_clearance_line(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def normalize(geometry: OptGeoT, **kwargs) -> OptGeoT: ...
@overload
def normalize(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def point_on_surface(geometry: Geometry, **kwargs) -> Point: ...
@overload
def point_on_surface(geometry: None, **kwargs) -> None: ...
@overload
def point_on_surface(geometry: Geometry | None, **kwargs) -> Point | None: ...
@overload
def point_on_surface(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def node(geometry: Geometry, **kwargs) -> MultiLineString: ...
@overload
def node(geometry: None, **kwargs) -> None: ...
@overload
def node(geometry: Geometry | None, **kwargs) -> MultiLineString | None: ...
@overload
def node(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def polygonize(geometries: Sequence[Geometry | None], **kwargs) -> GeometryCollection: ...
@overload
def polygonize(geometries: Sequence[Sequence[Geometry | None]], **kwargs) -> GeoArray: ...
@overload
def polygonize(geometries: OptGeoArrayLikeSeq, **kwargs) -> GeometryCollection | GeoArray: ...
@overload
def polygonize_full(
    geometries: Sequence[Geometry | None], **kwargs
) -> tuple[GeometryCollection, GeometryCollection, GeometryCollection, GeometryCollection]: ...
@overload
def polygonize_full(
    geometries: Sequence[Sequence[Geometry | None]], **kwargs
) -> tuple[GeoArray, GeoArray, GeoArray, GeoArray]: ...
@overload
def polygonize_full(
    geometries: OptGeoArrayLikeSeq, **kwargs
) -> (
    tuple[GeometryCollection, GeometryCollection, GeometryCollection, GeometryCollection]
    | tuple[GeoArray, GeoArray, GeoArray, GeoArray]
): ...
@overload
def remove_repeated_points(geometry: OptGeoT, tolerance: float = 0.0, **kwargs) -> OptGeoT: ...
@overload
def remove_repeated_points(geometry: OptGeoArrayLikeSeq, tolerance: float = 0.0, **kwargs) -> GeoArray: ...
@overload
def reverse(geometry: OptGeoT, **kwargs) -> OptGeoT: ...
@overload
def reverse(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def segmentize(geometry: OptGeoT, max_segment_length: float, **kwargs) -> OptGeoT: ...
@overload
def segmentize(geometry: OptGeoArrayLike, max_segment_length: ArrayLikeSeq[float], **kwargs) -> GeoArray: ...
@overload
def segmentize(geometry: OptGeoArrayLikeSeq, max_segment_length: ArrayLike[float], **kwargs) -> GeoArray: ...
@overload
def simplify(geometry: OptGeoT, tolerance: float, preserve_topology: bool = True, **kwargs) -> OptGeoT: ...
@overload
def simplify(geometry: OptGeoArrayLike, tolerance: ArrayLikeSeq[float], preserve_topology: bool = True, **kwargs) -> GeoArray: ...
@overload
def simplify(geometry: OptGeoArrayLikeSeq, tolerance: ArrayLike[float], preserve_topology: bool = True, **kwargs) -> GeoArray: ...
@overload
def snap(geometry: OptGeoT, reference: Geometry, tolerance: float, **kwargs) -> OptGeoT: ...
@overload
def snap(geometry: Geometry | None, reference: None, tolerance: float, **kwargs) -> None: ...
@overload
def snap(geometry: OptGeoArrayLikeSeq, reference: OptGeoArrayLike, tolerance: ArrayLike[float], **kwargs) -> GeoArray: ...
@overload
def snap(geometry: OptGeoArrayLike, reference: OptGeoArrayLikeSeq, tolerance: ArrayLike[float], **kwargs) -> GeoArray: ...
@overload
def snap(geometry: OptGeoArrayLike, reference: OptGeoArrayLike, tolerance: ArrayLikeSeq[float], **kwargs) -> GeoArray: ...
@overload
def voronoi_polygons(
    geometry: Geometry,
    tolerance: float = 0.0,
    extend_to: Geometry | None = None,
    only_edges: Literal[False] = False,
    ordered: bool = False,
    **kwargs,
) -> GeometryCollection[Polygon]: ...
@overload
def voronoi_polygons(
    geometry: Geometry, tolerance: float, extend_to: Geometry | None, only_edges: Literal[True], ordered: bool = False, **kwargs
) -> LineString | MultiLineString: ...
@overload
def voronoi_polygons(
    geometry: Geometry,
    tolerance: float = 0.0,
    extend_to: Geometry | None = None,
    *,
    only_edges: Literal[True],
    ordered: bool = False,
    **kwargs,
) -> LineString | MultiLineString: ...
@overload
def voronoi_polygons(
    geometry: Geometry,
    tolerance: float = 0.0,
    extend_to: Geometry | None = None,
    only_edges: bool = False,
    ordered: bool = False,
    **kwargs,
) -> GeometryCollection[Polygon] | LineString | MultiLineString: ...
@overload
def voronoi_polygons(
    geometry: None,
    tolerance: float = 0.0,
    extend_to: Geometry | None = None,
    only_edges: bool = False,
    ordered: bool = False,
    **kwargs,
) -> None: ...
@overload
def voronoi_polygons(
    geometry: Geometry | None,
    tolerance: float = 0.0,
    extend_to: Geometry | None = None,
    only_edges: bool = False,
    ordered: bool = False,
    **kwargs,
) -> GeometryCollection[Polygon] | LineString | MultiLineString | None: ...
@overload  # `geometry` as sequence-like
def voronoi_polygons(
    geometry: OptGeoArrayLikeSeq,
    tolerance: ArrayLike[float] = 0.0,
    extend_to: OptGeoArrayLike = None,
    only_edges: ArrayLike[bool] = False,
    ordered: ArrayLike[bool] = False,
    **kwargs,
) -> GeoArray: ...
@overload  # `tolerance` as sequence-like
def voronoi_polygons(
    geometry: OptGeoArrayLike,
    tolerance: ArrayLikeSeq[float],
    extend_to: OptGeoArrayLike = None,
    only_edges: ArrayLike[bool] = False,
    ordered: ArrayLike[bool] = False,
    **kwargs,
) -> GeoArray: ...
@overload
def voronoi_polygons(  # `extend_to` as positional sequence-like
    geometry: OptGeoArrayLike,
    tolerance: ArrayLike[float],
    extend_to: OptGeoArrayLikeSeq,
    only_edges: ArrayLike[bool] = False,
    ordered: ArrayLike[bool] = False,
    **kwargs,
) -> GeoArray: ...
@overload  # `extend_to` as keyword sequence-like
def voronoi_polygons(
    geometry: OptGeoArrayLike,
    tolerance: ArrayLike[float] = 0.0,
    *,
    extend_to: OptGeoArrayLikeSeq,
    only_edges: ArrayLike[bool] = False,
    ordered: ArrayLike[bool] = False,
    **kwargs,
) -> GeoArray: ...
@overload
def voronoi_polygons(  # `only_edges` as positional sequence-like
    geometry: OptGeoArrayLike,
    tolerance: ArrayLike[float],
    extend_to: OptGeoArrayLike,
    only_edges: ArrayLikeSeq[bool],
    ordered: ArrayLike[bool] = False,
    **kwargs,
) -> GeoArray: ...
@overload
def voronoi_polygons(  # `only_edges` as keyword sequence-like
    geometry: OptGeoArrayLike,
    tolerance: ArrayLike[float] = 0.0,
    extend_to: OptGeoArrayLike = None,
    *,
    only_edges: ArrayLikeSeq[bool],
    ordered: ArrayLike[bool] = False,
    **kwargs,
) -> GeoArray: ...
@overload  # `ordered` as positional sequence-like
def voronoi_polygons(
    geometry: OptGeoArrayLike,
    tolerance: ArrayLike[float],
    extend_to: OptGeoArrayLike,
    only_edges: ArrayLike[bool],
    ordered: ArrayLikeSeq[bool],
    **kwargs,
) -> GeoArray: ...
@overload  # `ordered` as keyword sequence-like
def voronoi_polygons(
    geometry: OptGeoArrayLike,
    tolerance: ArrayLike[float] = 0.0,
    extend_to: OptGeoArrayLike = None,
    *,
    only_edges: ArrayLike[bool] = False,
    ordered: ArrayLikeSeq[bool],
    **kwargs,
) -> GeoArray: ...
@overload
def oriented_envelope(geometry: Point, **kwargs) -> Point: ...
@overload
def oriented_envelope(geometry: Geometry, **kwargs) -> BaseGeometry: ...
@overload
def oriented_envelope(geometry: None, **kwargs) -> None: ...
@overload
def oriented_envelope(geometry: Geometry | None, **kwargs) -> BaseGeometry | None: ...
@overload
def oriented_envelope(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...

minimum_rotated_rectangle = oriented_envelope

@overload
def minimum_bounding_circle(geometry: Point, **kwargs) -> Point: ...
@overload
def minimum_bounding_circle(geometry: LineString | Polygon | BaseMultipartGeometry, **kwargs) -> Polygon: ...
@overload
def minimum_bounding_circle(geometry: Geometry, **kwargs) -> Polygon | Point: ...
@overload
def minimum_bounding_circle(geometry: None, **kwargs) -> None: ...
@overload
def minimum_bounding_circle(geometry: Geometry | None, **kwargs) -> Polygon | Point | None: ...
@overload
def minimum_bounding_circle(geometry: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def maximum_inscribed_circle(geometry: Polygon | MultiPolygon, tolerance: float | None = None, **kwargs) -> LineString: ...
@overload
def maximum_inscribed_circle(geometry: None, tolerance: float | None = None, **kwargs) -> None: ...
@overload
def maximum_inscribed_circle(
    geometry: Polygon | MultiPolygon | None, tolerance: float | None = None, **kwargs
) -> LineString | None: ...
@overload
def maximum_inscribed_circle(geometry: OptGeoArrayLikeSeq, tolerance: ArrayLike[float] | None = None, **kwargs) -> GeoArray: ...
@overload
def maximum_inscribed_circle(geometry: OptGeoArrayLike, tolerance: ArrayLikeSeq[float], **kwargs) -> GeoArray: ...
@overload
def orient_polygons(geometry: OptGeoT, *, exterior_cw: bool = False, **kwargs) -> OptGeoT: ...
@overload
def orient_polygons(geometry: OptGeoArrayLikeSeq, *, exterior_cw: bool = False, **kwargs) -> GeoArray: ...



from collections.abc import Sequence
from typing import Any, Literal, overload

from tensorflow import RaggedTensor, Tensor
from tensorflow._aliases import ScalarTensorCompatible, TensorCompatible, TensorOrArray
from tensorflow.math import l2_normalize as l2_normalize, sigmoid as sigmoid, tanh as tanh
from tensorflow.sparse import SparseTensor

def atrous_conv2d(
    value: TensorOrArray, filters: TensorOrArray, rate: int, padding: Literal["VALID", "SAME"], name: str | None = None
) -> Tensor: ...
def atrous_conv2d_transpose(
    value: TensorOrArray,
    filters: TensorOrArray,
    output_shape: TensorOrArray,
    rate: int,
    padding: Literal["VALID", "SAME"],
    name: str | None = None,
) -> Tensor: ...
def avg_pool(
    input: TensorOrArray,
    ksize: int | Sequence[int],
    strides: int | Sequence[int],
    padding: Literal["VALID", "SAME"],
    data_format: Literal["NWC", "NCW", "NHWC", "NCHW", "NDHWC", "NCDHW"] | None = None,
    name: str | None = None,
) -> Tensor: ...
def avg_pool1d(
    input: TensorOrArray,
    ksize: int | Sequence[int],
    strides: int | Sequence[int],
    padding: Literal["VALID", "SAME"],
    data_format: Literal["NWC", "NCW"] = "NWC",
    name: str | None = None,
) -> Tensor: ...
def avg_pool2d(
    input: TensorOrArray,
    ksize: int | Sequence[int],
    strides: int | Sequence[int],
    padding: Literal["VALID", "SAME"],
    data_format: Literal["NHWC", "NCHW"] = "NHWC",
    name: str | None = None,
) -> Tensor: ...
def avg_pool3d(
    input: TensorOrArray,
    ksize: int | Sequence[int],
    strides: int | Sequence[int],
    padding: Literal["VALID", "SAME"],
    data_format: Literal["NDHWC", "NCDHW"] = "NDHWC",
    name: str | None = None,
) -> Tensor: ...
def batch_normalization(
    x: Tensor, mean: Tensor, variance: Tensor, offset: Tensor, scale: Tensor, variance_epsilon: float, name: str | None = None
) -> Tensor: ...
def bias_add(
    value: Tensor, bias: Tensor, data_format: Literal["N...C", "NC..."] | None = None, name: str | None = None
) -> Tensor: ...
def collapse_repeated(labels: Tensor, seq_length: Tensor, name: str | None = None) -> tuple[Tensor, Tensor]: ...
def compute_accidental_hits(
    true_classes: Tensor, sampled_candidates: Tensor, num_true: int, seed: int | None = None, name: str | None = None
) -> tuple[Tensor, Tensor, Tensor]: ...
def compute_average_loss(
    per_example_loss: Tensor, sample_weight: Tensor | None = None, global_batch_size: int | None = None
) -> Tensor: ...
def conv1d(
    input: TensorOrArray,
    filters: TensorOrArray,
    stride: int | Sequence[int],
    padding: Literal["VALID", "SAME"],
    data_format: Literal["NWC", "NCW"] = "NWC",
    dilations: int | Sequence[int] | None = None,
    name: str | None = None,
) -> Tensor: ...
def conv1d_transpose(
    input: TensorOrArray,
    filters: TensorOrArray,
    output_shape: TensorOrArray,
    strides: int | Sequence[int],
    padding: Literal["VALID", "SAME"] = "SAME",
    data_format: Literal["NWC", "NCW"] = "NWC",
    dilations: int | Sequence[int] | None = None,
    name: str | None = None,
) -> Tensor: ...
def conv2d(
    input: TensorOrArray,
    filters: TensorOrArray,
    strides: int | Sequence[int],
    padding: Literal["VALID", "SAME"],
    data_format: Literal["NHWC", "NCHW"] = "NHWC",
    dilations: int | Sequence[int] | None = None,
    name: str | None = None,
) -> Tensor: ...
def conv2d_transpose(
    input: TensorOrArray,
    filters: TensorOrArray,
    output_shape: TensorOrArray,
    strides: int | Sequence[int],
    padding: Literal["VALID", "SAME"] = "SAME",
    data_format: Literal["NHWC", "NCHW"] = "NHWC",
    dilations: int | Sequence[int] | None = None,
    name: str | None = None,
) -> Tensor: ...
def conv3d(
    input: TensorOrArray,
    filters: TensorOrArray,
    strides: int | Sequence[int],
    padding: Literal["VALID", "SAME"],
    data_format: Literal["NDHWC", "NCDHW"] = "NDHWC",
    dilations: int | Sequence[int] | None = None,
    name: str | None = None,
) -> Tensor: ...
def conv3d_transpose(
    input: TensorOrArray,
    filters: TensorOrArray,
    output_shape: TensorOrArray,
    strides: int | Sequence[int],
    padding: Literal["VALID", "SAME"] = "SAME",
    data_format: Literal["NDHWC", "NCDHW"] = "NDHWC",
    dilations: int | Sequence[int] | None = None,
    name: str | None = None,
) -> Tensor: ...
def conv_transpose(
    input: TensorOrArray,
    filters: TensorOrArray,
    output_shape: TensorOrArray,
    strides: int | Sequence[int],
    padding: Literal["VALID", "SAME"] = "SAME",
    data_format: str | None = None,
    dilations: int | Sequence[int] | None = None,
    name: str | None = None,
) -> Tensor: ...
def convolution(
    input: TensorOrArray,
    filters: TensorOrArray,
    strides: int | Sequence[int] | None = None,
    padding: Literal["VALID", "SAME"] = "VALID",
    data_format: Literal["NC", "NWC", "NCW", "NHWC", "NCHW", "NDHWC", "NCDHW"] | None = None,
    dilations: int | Sequence[int] | None = None,
    name: str | None = None,
) -> Tensor: ...
def crelu(features: TensorOrArray, axis: int = -1, name: str | None = None) -> Tensor: ...
def ctc_beam_search_decoder(
    inputs: TensorOrArray, sequence_length: TensorOrArray | Sequence[int], beam_width: int = 100, top_paths: int = 1
) -> tuple[list[SparseTensor], Tensor]: ...
def ctc_greedy_decoder(
    inputs: Tensor, sequence_length: Tensor | Sequence[int], merge_repeated: bool = True, blank_index: int | None = None
) -> tuple[list[SparseTensor], Tensor]: ...
def ctc_loss(
    labels: Tensor,
    logits: Tensor,
    label_length: Tensor,
    logit_length: Tensor,
    logits_time_major: bool = True,
    unique: int | None = None,
    blank_index: int | None = None,
    name: str | None = None,
) -> Tensor: ...
def ctc_unique_labels(labels: Tensor, name: str | None = None) -> tuple[Tensor, Tensor]: ...
@overload
def embedding_lookup(
    params: TensorCompatible, ids: TensorCompatible, max_norm: float | None = None, name: str | None = None
) -> Tensor: ...
@overload
def embedding_lookup(
    params: TensorCompatible, ids: RaggedTensor, max_norm: float | None = None, name: str | None = None
) -> RaggedTensor: ...
def leaky_relu(features: TensorCompatible, alpha: float = 0.2, name: str | None = None) -> Tensor: ...
def log_poisson_loss(
    targets: TensorCompatible, log_input: TensorCompatible, compute_full_loss: bool = False, name: str | None = None
) -> Tensor: ...

# tf.nn.moments's shift is not used in the current implementation.
def moments(
    x: TensorCompatible | RaggedTensor,
    axes: TensorCompatible,
    shift: None | Any = None,
    keepdims: bool = False,
    name: str | None = None,
) -> tuple[Tensor, Tensor]: ...
def relu(features: TensorCompatible, name: str | None = None) -> Tensor: ...
def sigmoid_cross_entropy_with_logits(labels: TensorCompatible, logits: TensorCompatible, name: str | None = None) -> Tensor: ...
def softmax(logits: TensorCompatible, axis: ScalarTensorCompatible | None = None, name: str | None = None) -> Tensor: ...
def selu(features: TensorCompatible, name: str | None = None) -> Tensor: ...
def safe_embedding_lookup_sparse(
    embedding_weights: Tensor | list[Tensor],
    sparse_ids: SparseTensor,
    sparse_weights: SparseTensor | None = None,
    combiner: str = "mean",
    default_id: ScalarTensorCompatible | None = None,
    max_norm: float | None = None,
    name: str | None = None,
    allow_fast_lookup: bool = False,
) -> Tensor: ...
def __getattr__(name: str): ...  # incomplete module



from logging import Logger
from typing import NamedTuple

LOG: Logger

class Violation(NamedTuple):
    code: str
    filename: str
    line_number: int
    column_number: int
    text: str
    physical_line: str | None
    def is_inline_ignored(self, disable_noqa: bool) -> bool: ...



from .build import (
    create_archive as create_archive,
    exclude_paths as exclude_paths,
    match_tag as match_tag,
    mkbuildcontext as mkbuildcontext,
    tar as tar,
)
from .decorators import check_resource as check_resource, minimum_version as minimum_version, update_headers as update_headers
from .utils import (
    compare_version as compare_version,
    convert_filters as convert_filters,
    convert_port_bindings as convert_port_bindings,
    convert_service_networks as convert_service_networks,
    convert_volume_binds as convert_volume_binds,
    create_host_config as create_host_config,
    create_ipam_config as create_ipam_config,
    create_ipam_pool as create_ipam_pool,
    datetime_to_timestamp as datetime_to_timestamp,
    decode_json_header as decode_json_header,
    format_environment as format_environment,
    format_extra_hosts as format_extra_hosts,
    kwargs_from_env as kwargs_from_env,
    normalize_links as normalize_links,
    parse_bytes as parse_bytes,
    parse_devices as parse_devices,
    parse_env_file as parse_env_file,
    parse_host as parse_host,
    parse_repository_tag as parse_repository_tag,
    split_command as split_command,
    version_gte as version_gte,
    version_lt as version_lt,
)



from __future__ import annotations

import mmap
import re
from typing import AnyStr, Match, Optional
from typing_extensions import assert_type


def check_search(str_pat: re.Pattern[str], bytes_pat: re.Pattern[bytes]) -> None:
    assert_type(str_pat.search("x"), Optional[Match[str]])
    assert_type(bytes_pat.search(b"x"), Optional[Match[bytes]])
    assert_type(bytes_pat.search(bytearray(b"x")), Optional[Match[bytes]])
    assert_type(bytes_pat.search(mmap.mmap(0, 10)), Optional[Match[bytes]])


def check_search_with_AnyStr(pattern: re.Pattern[AnyStr], string: AnyStr) -> re.Match[AnyStr]:
    """See issue #9591"""
    match = pattern.search(string)
    if match is None:
        raise ValueError(f"'{string!r}' does not match {pattern!r}")
    return match


def check_no_ReadableBuffer_false_negatives() -> None:
    re.compile("foo").search(bytearray(b"foo"))  # type: ignore
    re.compile("foo").search(mmap.mmap(0, 10))  # type: ignore



from m3u8.version_matching_rules import VersionMatchingError

def get_version(file_lines: list[str]) -> float | None: ...
def valid_in_all_rules(line_number: int, line: str, version: float) -> list[VersionMatchingError]: ...
def validate(file_lines: list[str]) -> list[VersionMatchingError]: ...



from _typeshed import Incomplete, SupportsWrite
from collections import defaultdict
from collections.abc import Callable, Container, Iterable
from types import ModuleType
from typing import Final, Literal
from typing_extensions import TypeAlias, TypeGuard

IS_INTERACTIVE: bool

__author__: Final[str]
__copyright__: Final[str]
__license__: Final[str]
__version__: Final[str]
__date__: Final[str]

# GraphViz has types, but does not include the py.typed file.
# See https://github.com/xflr6/graphviz/pull/180
_GraphvizSource: TypeAlias = Incomplete
_Filter: TypeAlias = Callable[[object], bool]

def count(typename: str, objects: Iterable[object] | None = None) -> int: ...
def typestats(
    objects: Iterable[object] | None = None, shortnames: bool = True, filter: _Filter | None = None
) -> dict[str, int]: ...
def most_common_types(
    limit: int = 10, objects: Iterable[object] | None = None, shortnames: bool = True, filter: _Filter | None = None
) -> list[tuple[str, int]]: ...
def show_most_common_types(
    limit: int = 10,
    objects: Iterable[object] | None = None,
    shortnames: bool = True,
    file: SupportsWrite[str] | None = None,
    filter: _Filter | None = None,
) -> None: ...
def growth(
    limit: int = 10, peak_stats: dict[str, int] = {}, shortnames: bool = True, filter: _Filter | None = None
) -> list[tuple[str, int, int]]: ...
def show_growth(
    limit: int = 10,
    peak_stats: dict[str, int] | None = None,
    shortnames: bool = True,
    file: SupportsWrite[str] | None = None,
    filter: _Filter | None = None,
) -> None: ...
def get_new_ids(
    skip_update: bool = False,
    limit: int = 10,
    sortby: Literal["old", "current", "new", "deltas"] = "deltas",
    shortnames: bool | None = None,
    file: SupportsWrite[str] | None = None,
) -> defaultdict[str, set[int]]: ...
def get_leaking_objects(objects: Iterable[object] | None = None) -> list[object]: ...
def by_type(typename: str, objects: Iterable[object] | None = None) -> list[object]: ...
def at(addr: int) -> object: ...
def at_addrs(address_set: Container[int]) -> list[object]: ...
def find_ref_chain(obj: object, predicate: _Filter, max_depth: int = 20, extra_ignore: Iterable[int] = ()) -> list[object]: ...
def find_backref_chain(
    obj: object, predicate: _Filter, max_depth: int = 20, extra_ignore: Iterable[int] = ()
) -> list[object]: ...
def show_backrefs(
    objs: object,
    max_depth: int = 3,
    extra_ignore: Iterable[int] = (),
    filter: _Filter | None = None,
    too_many: int = 10,
    highlight: object = None,
    filename: str | None = None,
    extra_info: Callable[[object], str] | None = None,
    refcounts: bool = False,
    shortnames: bool = True,
    output: SupportsWrite[str] | None = None,
    extra_node_attrs: Callable[[object], dict[str, str]] | None = None,
) -> None | _GraphvizSource: ...
def show_refs(
    objs: object,
    max_depth: int = 3,
    extra_ignore: Iterable[int] = (),
    filter: _Filter | None = None,
    too_many: int = 10,
    highlight: object = None,
    filename: str | None = None,
    extra_info: Callable[[object], str] | None = None,
    refcounts: bool = False,
    shortnames: bool = True,
    output: SupportsWrite[str] | None = None,
    extra_node_attrs: Callable[[object], dict[str, str]] | None = None,
) -> None | _GraphvizSource: ...
def show_chain(
    *chains: list[object], obj: object, predicate: _Filter, max_depth: int = 20, extra_ignore: Iterable[int] = ()
) -> None: ...
def is_proper_module(obj: object) -> TypeGuard[ModuleType]: ...



from typing import Final, Literal

from docutils.nodes import Node

__docformat__: Final = "reStructuredText"

class MathError(ValueError):
    details: list[Node]
    def __init__(self, msg: object, details: list[Node] = []) -> None: ...

def toplevel_code(code: str) -> str: ...
def pick_math_environment(code: str, numbered: bool = False) -> Literal["align*", "equation*", "align", "equation"]: ...
def wrap_math_code(code: str, as_block: bool | None) -> str: ...



from binascii import hexlify as hexlify
from typing import Final, Literal, overload

from passlib.handlers.windows import nthash as nthash

LM_MAGIC: Final[bytes]
raw_nthash = nthash.raw_nthash

@overload
def raw_lmhash(secret: str | bytes, encoding: str = "ascii", hex: Literal[False] = False) -> bytes: ...
@overload
def raw_lmhash(secret: str | bytes, encoding: str, hex: Literal[True]) -> str: ...
@overload
def raw_lmhash(secret: str | bytes, *, hex: Literal[True]) -> str: ...

__all__ = ["nthash", "raw_lmhash", "raw_nthash"]



from _typeshed import ConvertibleToInt, Incomplete, Unused
from re import Pattern
from typing import ClassVar, Final, Literal, SupportsIndex, overload
from typing_extensions import TypeGuard

from openpyxl.descriptors import Strict, String
from openpyxl.descriptors.base import Integer
from openpyxl.descriptors.serialisable import Serialisable

BUILTIN_FORMATS: Final[dict[int, str]]
BUILTIN_FORMATS_MAX_SIZE: Final = 164
BUILTIN_FORMATS_REVERSE: Final[dict[str, int]]
FORMAT_GENERAL: Final = "General"
FORMAT_TEXT: Final = "@"
FORMAT_NUMBER: Final = "0"
FORMAT_NUMBER_00: Final = "0.00"
FORMAT_NUMBER_COMMA_SEPARATED1: Final = "#,##0.00"
FORMAT_NUMBER_COMMA_SEPARATED2: Final = "#,##0.00_-"
FORMAT_PERCENTAGE: Final = "0%"
FORMAT_PERCENTAGE_00: Final = "0.00%"
FORMAT_DATE_YYYYMMDD2: Final = "yyyy-mm-dd"
FORMAT_DATE_YYMMDD: Final = "yy-mm-dd"
FORMAT_DATE_DDMMYY: Final = "dd/mm/yy"
FORMAT_DATE_DMYSLASH: Final = "d/m/y"
FORMAT_DATE_DMYMINUS: Final = "d-m-y"
FORMAT_DATE_DMMINUS: Final = "d-m"
FORMAT_DATE_MYMINUS: Final = "m-y"
FORMAT_DATE_XLSX14: Final = "mm-dd-yy"
FORMAT_DATE_XLSX15: Final = "d-mmm-yy"
FORMAT_DATE_XLSX16: Final = "d-mmm"
FORMAT_DATE_XLSX17: Final = "mmm-yy"
FORMAT_DATE_XLSX22: Final = "m/d/yy h:mm"
FORMAT_DATE_DATETIME: Final = "yyyy-mm-dd h:mm:ss"
FORMAT_DATE_TIME1: Final = "h:mm AM/PM"
FORMAT_DATE_TIME2: Final = "h:mm:ss AM/PM"
FORMAT_DATE_TIME3: Final = "h:mm"
FORMAT_DATE_TIME4: Final = "h:mm:ss"
FORMAT_DATE_TIME5: Final = "mm:ss"
FORMAT_DATE_TIME6: Final = "h:mm:ss"
FORMAT_DATE_TIME7: Final = "i:s.S"
FORMAT_DATE_TIME8: Final = "h:mm:ss@"
FORMAT_DATE_TIMEDELTA: Final = "[hh]:mm:ss"
FORMAT_DATE_YYMMDDSLASH: Final = "yy/mm/dd@"
FORMAT_CURRENCY_USD_SIMPLE: Final = '"$"#,##0.00_-'
FORMAT_CURRENCY_USD: Final = "$#,##0_-"
FORMAT_CURRENCY_EUR_SIMPLE: Final = "[$EUR ]#,##0.00_-"

COLORS: Final[str]
LITERAL_GROUP: Final = r'".*?"'
LOCALE_GROUP: Final = r"\[(?!hh?\]|mm?\]|ss?\])[^\]]*\]"
STRIP_RE: Final[Pattern[str]]
TIMEDELTA_RE: Final[Pattern[str]]

def is_date_format(fmt: str | None) -> TypeGuard[str]: ...
def is_timedelta_format(fmt: str | None) -> TypeGuard[str]: ...
@overload
def is_datetime(fmt: None) -> None: ...
@overload
def is_datetime(fmt: str) -> Literal["datetime", "date", "time"] | None: ...
def is_builtin(fmt: str | None) -> TypeGuard[str]: ...
def builtin_format_code(index: int) -> str | None: ...
@overload
def builtin_format_id(fmt: None) -> None: ...
@overload
def builtin_format_id(fmt: str) -> int | None: ...

class NumberFormatDescriptor(String[Incomplete]):
    def __set__(self, instance: Serialisable | Strict, value) -> None: ...

class NumberFormat(Serialisable):
    numFmtId: Integer[Literal[False]]
    formatCode: String[Literal[False]]
    def __init__(self, numFmtId: ConvertibleToInt, formatCode: str) -> None: ...

class NumberFormatList(Serialisable):
    # Overwritten by property below
    # count: Integer
    numFmt: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    __attrs__: ClassVar[tuple[str, ...]]
    def __init__(self, count: Unused = None, numFmt=()) -> None: ...
    @property
    def count(self) -> int: ...
    def __getitem__(self, idx: SupportsIndex) -> NumberFormat: ...



import datetime
from typing import Final

from ..core import ChineseNewYearCalendar

holidays: Final[dict[int, dict[str, list[tuple[int, int]]]]]
workdays: Final[dict[int, dict[str, list[tuple[int, int]]]]]

class China(ChineseNewYearCalendar):
    extra_working_days: list[datetime.date]
    def __init__(self, *args, **kwargs) -> None: ...



import datetime

from ..core import WesternCalendar

class Lithuania(WesternCalendar):
    def get_mothers_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_fathers_day(self, year: int) -> tuple[datetime.date, str]: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["average_clustering"]

@_dispatchable
def average_clustering(G: Graph[_Node], trials: int = 1000, seed: int | RandomState | None = None) -> float: ...



from typing import TextIO

from antlr4.atn.LexerATNSimulator import LexerATNSimulator as LexerATNSimulator
from antlr4.CommonTokenFactory import CommonTokenFactory as CommonTokenFactory
from antlr4.error.Errors import (
    IllegalStateException as IllegalStateException,
    LexerNoViableAltException as LexerNoViableAltException,
    RecognitionException as RecognitionException,
)
from antlr4.InputStream import InputStream as InputStream
from antlr4.Recognizer import Recognizer as Recognizer
from antlr4.Token import CommonToken, Token as Token

class TokenSource: ...

class Lexer(Recognizer, TokenSource):
    __slots__ = (
        "_input",
        "_output",
        "_factory",
        "_tokenFactorySourcePair",
        "_token",
        "_tokenStartCharIndex",
        "_tokenStartLine",
        "_tokenStartColumn",
        "_hitEOF",
        "_channel",
        "_type",
        "_modeStack",
        "_mode",
        "_text",
    )
    DEFAULT_MODE: int
    MORE: int
    SKIP: int
    DEFAULT_TOKEN_CHANNEL: int
    HIDDEN: int
    MIN_CHAR_VALUE: int
    MAX_CHAR_VALUE: int
    _input: InputStream
    _output: TextIO
    _factory: CommonTokenFactory
    _tokenFactorySourcePair: tuple[TokenSource, InputStream]
    _interp: LexerATNSimulator
    _token: Token | None
    _tokenStartCharIndex: int
    _tokenStartLine: int
    _tokenStartColumn: int
    _hitEOF: bool
    _channel: int
    _type: int
    _modeStack: list[int]
    _mode: int
    _text: str | None
    def __init__(self, input: InputStream, output: TextIO = ...) -> None: ...
    def reset(self) -> None: ...
    def nextToken(self) -> Token | None: ...
    def skip(self) -> None: ...
    def more(self) -> None: ...
    def mode(self, m: int) -> None: ...
    def pushMode(self, m: int) -> None: ...
    def popMode(self) -> int: ...
    @property
    def inputStream(self) -> InputStream: ...
    @inputStream.setter
    def inputStream(self, input: InputStream) -> None: ...
    @property
    def sourceName(self) -> str: ...
    def emitToken(self, token: Token) -> None: ...
    def emit(self) -> CommonToken: ...
    def emitEOF(self) -> CommonToken: ...
    @property
    def type(self) -> int: ...
    @type.setter
    def type(self, type: int) -> None: ...
    @property
    def line(self) -> int: ...
    @line.setter
    def line(self, line: int) -> None: ...
    @property
    def column(self) -> int: ...
    @column.setter
    def column(self, column: int) -> None: ...
    def getCharIndex(self) -> int: ...
    @property
    def text(self) -> str: ...
    @text.setter
    def text(self, txt: str) -> None: ...
    def getAllTokens(self) -> list[Token]: ...
    def notifyListeners(self, e: LexerNoViableAltException) -> None: ...
    def getErrorDisplay(self, s: str) -> str: ...
    def getErrorDisplayForChar(self, c: str) -> str: ...
    def getCharErrorDisplay(self, c: str) -> str: ...
    def recover(self, re: RecognitionException) -> None: ...



from collections.abc import Generator, Iterable

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "number_strongly_connected_components",
    "strongly_connected_components",
    "is_strongly_connected",
    "kosaraju_strongly_connected_components",
    "condensation",
]

@_dispatchable
def strongly_connected_components(G: DiGraph[_Node]) -> Generator[set[_Node]]: ...
@_dispatchable
def kosaraju_strongly_connected_components(G: DiGraph[_Node], source: _Node | None = None) -> Generator[set[_Node]]: ...
@_dispatchable
def number_strongly_connected_components(G: DiGraph[_Node]) -> int: ...
@_dispatchable
def is_strongly_connected(G: DiGraph[_Node]) -> bool: ...
@_dispatchable
def condensation(G: DiGraph[_Node], scc: Iterable[Iterable[_Node]] | None = None) -> DiGraph[int]: ...



from typing import Any

from .._types import _WSGIAppType

def get_wsgi_app(config_uri: str, name: str | None = None, defaults: dict[str, Any] | None = None) -> _WSGIAppType: ...
def has_logging_config(config_file: str) -> bool: ...
def serve(app: _WSGIAppType, global_conf: dict[str, Any], **local_conf: Any) -> None: ...



from _typeshed import Incomplete
from collections.abc import Hashable, Iterable, Mapping, Sequence
from typing import Literal, TypedDict, type_check_only
from typing_extensions import Self, TypeAlias

import numpy as np
import pandas as pd
from matplotlib.axes import Axes
from matplotlib.colors import Colormap, ListedColormap, Normalize
from matplotlib.gridspec import GridSpec
from matplotlib.typing import ColorType
from numpy._typing import _ArrayLikeInt_co
from numpy.typing import ArrayLike, NDArray

from .axisgrid import Grid

# pandas._typing.ListLikeU is partially Unknown
_ListLikeU: TypeAlias = Sequence[Incomplete] | NDArray[Incomplete] | pd.Series[Incomplete] | pd.Index[Incomplete]
_ConvertibleToDataFrame: TypeAlias = (
    _ListLikeU
    | pd.DataFrame
    | dict[Incomplete, Incomplete]
    | Iterable[_ListLikeU | tuple[Hashable, _ListLikeU] | dict[Incomplete, Incomplete]]
    | None
)
_FlatOrNestedSequenceOfColors: TypeAlias = (
    Sequence[ColorType]
    | Sequence[Iterable[ColorType]]
    | NDArray[Incomplete]
    | pd.Index[Incomplete]
    | pd.Series[Incomplete]
    | pd.DataFrame
)

__all__ = ["heatmap", "clustermap"]

def heatmap(
    data: pd.DataFrame | ArrayLike,
    *,
    vmin: float | None = None,
    vmax: float | None = None,
    cmap: str | list[ColorType] | Colormap | None = None,
    center: float | None = None,
    robust: bool = False,
    annot: bool | ArrayLike | None = None,
    fmt: str = ".2g",
    annot_kws: dict[str, Incomplete] | None = None,
    linewidths: float = 0,
    linecolor: ColorType = "white",
    cbar: bool = True,
    cbar_kws: dict[str, Incomplete] | None = None,
    cbar_ax: Axes | None = None,
    square: bool = False,
    xticklabels: Literal["auto"] | bool | int | Sequence[str] = "auto",
    yticklabels: Literal["auto"] | bool | int | Sequence[str] = "auto",
    mask: NDArray[np.bool_] | pd.DataFrame | None = None,
    ax: Axes | None = None,
    # Kwargs below passed to matplotlib.axes.Axes.pcolormesh
    alpha: float | None = None,
    norm: str | Normalize | None = None,
    shading: Literal["flat", "nearest", "gouraud", "auto"] | None = None,
    antialiased: bool = False,
    **kwargs,
) -> Axes: ...
@type_check_only
class _Dendogram(TypedDict):
    icoord: list[list[float]]
    dcoord: list[list[float]]
    ivl: list[str]
    leaves: list[int]
    color_list: list[str]
    leaves_color_list: list[str]

class _DendrogramPlotter:
    axis: int
    array: NDArray[np.floating]
    data: pd.DataFrame
    shape: tuple[int, int]
    metric: str
    method: str
    label: bool
    rotate: bool
    linkage: NDArray[np.floating]
    dendrogram: _Dendogram
    xticks: list[float] | NDArray[np.floating]
    yticks: list[float] | NDArray[np.floating]
    xticklabels: list[str]
    yticklabels: list[str]
    ylabel: str
    xlabel: str
    dependent_coord: list[list[float]]
    independent_coord: list[list[float]]
    def __init__(
        self,
        data: pd.DataFrame,
        linkage: NDArray[np.floating] | None,
        metric: str,
        method: str,
        axis: int,
        label: bool,
        rotate: bool,
    ) -> None: ...
    @property
    def calculated_linkage(self) -> NDArray[np.float64]: ...
    def calculate_dendrogram(self) -> _Dendogram: ...
    @property
    def reordered_ind(self) -> list[int]: ...
    def plot(self, ax: Axes, tree_kws: dict[str, Incomplete]) -> Self: ...

def dendrogram(
    data: pd.DataFrame,
    *,
    linkage: NDArray[np.floating] | None = None,
    axis: int = 1,
    label: bool = True,
    metric: str = "euclidean",
    method: str = "average",
    rotate: bool = False,
    tree_kws: dict[str, Incomplete] | None = None,
    ax: Axes | None = None,
) -> _DendrogramPlotter: ...

class ClusterGrid(Grid):
    data: pd.DataFrame
    data2d: pd.DataFrame
    mask: pd.DataFrame
    row_colors: list[list[tuple[float, float, float]]] | None
    row_color_labels: list[str] | None
    col_colors: list[list[tuple[float, float, float]]] | None
    col_color_labels: list[str] | None
    gs: GridSpec
    ax_row_dendrogram: Axes
    ax_col_dendrogram: Axes
    ax_row_colors: Axes | None
    ax_col_colors: Axes | None
    ax_heatmap: Axes
    ax_cbar: Axes | None
    cax: Axes | None
    cbar_pos: tuple[float, float, float, float] | None
    dendrogram_row: _DendrogramPlotter | None
    dendrogram_col: _DendrogramPlotter | None
    def __init__(
        self,
        data: _ConvertibleToDataFrame,
        pivot_kws: Mapping[str, Incomplete] | None = None,
        z_score: int | None = None,
        standard_scale: int | None = None,
        figsize: tuple[float, float] | None = None,
        row_colors: _FlatOrNestedSequenceOfColors | None = None,
        col_colors: _FlatOrNestedSequenceOfColors | None = None,
        mask: NDArray[np.bool_] | pd.DataFrame | None = None,
        dendrogram_ratio: float | tuple[float, float] | None = None,
        colors_ratio: float | tuple[float, float] | None = None,
        cbar_pos: tuple[float, float, float, float] | None = None,
    ) -> None: ...
    def format_data(
        self,
        data: pd.DataFrame,
        pivot_kws: Mapping[str, Incomplete] | None,
        z_score: int | None = None,
        standard_scale: int | None = None,
    ) -> pd.DataFrame: ...
    @staticmethod
    def z_score(data2d: pd.DataFrame, axis: int = 1) -> pd.DataFrame: ...
    @staticmethod
    def standard_scale(data2d: pd.DataFrame, axis: int = 1) -> pd.DataFrame: ...
    def dim_ratios(self, colors: ArrayLike | None, dendrogram_ratio: float, colors_ratio: float) -> list[float]: ...
    @staticmethod
    def color_list_to_matrix_and_cmap(
        colors: _FlatOrNestedSequenceOfColors, ind: _ArrayLikeInt_co, axis: int = 0
    ) -> tuple[NDArray[np.int_], ListedColormap]: ...
    def plot_dendrograms(
        self,
        row_cluster: bool,
        col_cluster: bool,
        metric: str,
        method: str,
        row_linkage: NDArray[np.floating] | None,
        col_linkage: NDArray[np.floating] | None,
        tree_kws: dict[str, Incomplete] | None,
    ) -> None: ...
    def plot_colors(self, xind: _ArrayLikeInt_co, yind: _ArrayLikeInt_co, **kws) -> None: ...
    def plot_matrix(self, colorbar_kws: dict[str, Incomplete], xind: _ArrayLikeInt_co, yind: _ArrayLikeInt_co, **kws) -> None: ...
    def plot(
        self,
        metric: str,
        method: str,
        colorbar_kws: dict[str, Incomplete] | None,
        row_cluster: bool,
        col_cluster: bool,
        row_linkage: NDArray[np.floating] | None,
        col_linkage: NDArray[np.floating] | None,
        tree_kws: dict[str, Incomplete] | None,
        **kws,
    ) -> Self: ...

def clustermap(
    data: _ConvertibleToDataFrame,
    *,
    pivot_kws: dict[str, Incomplete] | None = None,
    method: str = "average",
    metric: str = "euclidean",
    z_score: int | None = None,
    standard_scale: int | None = None,
    figsize: tuple[float, float] | None = (10, 10),
    cbar_kws: dict[str, Incomplete] | None = None,
    row_cluster: bool = True,
    col_cluster: bool = True,
    row_linkage: NDArray[np.floating] | None = None,
    col_linkage: NDArray[np.floating] | None = None,
    row_colors: _FlatOrNestedSequenceOfColors | None = None,
    col_colors: _FlatOrNestedSequenceOfColors | None = None,
    mask: NDArray[np.bool_] | pd.DataFrame | None = None,
    dendrogram_ratio: float | tuple[float, float] = 0.2,
    colors_ratio: float | tuple[float, float] = 0.03,
    cbar_pos: tuple[float, float, float, float] | None = (0.02, 0.8, 0.05, 0.18),
    tree_kws: dict[str, Incomplete] | None = None,
    **kwargs,
) -> ClusterGrid: ...



import abc

import pika.compat

class AbstractIOServices(pika.compat.AbstractBase, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def get_native_ioloop(self): ...
    @abc.abstractmethod
    def close(self): ...
    @abc.abstractmethod
    def run(self): ...
    @abc.abstractmethod
    def stop(self): ...
    @abc.abstractmethod
    def add_callback_threadsafe(self, callback): ...
    @abc.abstractmethod
    def call_later(self, delay, callback): ...
    @abc.abstractmethod
    def getaddrinfo(self, host, port, on_done, family: int = 0, socktype: int = 0, proto: int = 0, flags: int = 0): ...
    @abc.abstractmethod
    def connect_socket(self, sock, resolved_addr, on_done): ...
    @abc.abstractmethod
    def create_streaming_connection(self, protocol_factory, sock, on_done, ssl_context=None, server_hostname=None): ...

class AbstractFileDescriptorServices(pika.compat.AbstractBase, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def set_reader(self, fd, on_readable): ...
    @abc.abstractmethod
    def remove_reader(self, fd): ...
    @abc.abstractmethod
    def set_writer(self, fd, on_writable): ...
    @abc.abstractmethod
    def remove_writer(self, fd): ...

class AbstractTimerReference(pika.compat.AbstractBase, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def cancel(self): ...

class AbstractIOReference(pika.compat.AbstractBase, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def cancel(self): ...

class AbstractStreamProtocol(pika.compat.AbstractBase, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def connection_made(self, transport): ...
    @abc.abstractmethod
    def connection_lost(self, error): ...
    @abc.abstractmethod
    def eof_received(self): ...
    @abc.abstractmethod
    def data_received(self, data): ...

class AbstractStreamTransport(pika.compat.AbstractBase, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def abort(self): ...
    @abc.abstractmethod
    def get_protocol(self): ...
    @abc.abstractmethod
    def write(self, data): ...
    @abc.abstractmethod
    def get_write_buffer_size(self): ...



from _typeshed import SupportsRead
from collections.abc import Iterator
from io import StringIO
from typing import IO, Any, Literal, TypedDict, overload, type_check_only
from typing_extensions import TypeAlias

from docker._types import JSON

from .resource import Collection, Model

_ImageList: TypeAlias = list[Image]  # To resolve conflicts with a method called "list"

@type_check_only
class _ContainerLimits(TypedDict, total=False):
    memory: int
    memswap: int
    cpushares: int
    cpusetcpus: str

class Image(Model):
    @property
    def labels(self) -> dict[str, Any]: ...
    @property
    def short_id(self) -> str: ...
    @property
    def tags(self) -> list[str]: ...
    def history(self) -> list[Any]: ...
    def remove(self, force: bool = False, noprune: bool = False) -> dict[str, Any]: ...
    def save(self, chunk_size: int = 2097152, named: bool = False) -> Iterator[Any]: ...
    def tag(self, repository: str, tag: str | None = None, **kwargs) -> bool: ...

class RegistryData(Model):
    image_name: str
    def __init__(self, image_name: str, *args, **kwargs) -> None: ...
    @property
    def id(self) -> str: ...
    @property
    def short_id(self) -> str: ...
    def pull(self, platform: str | None = None) -> Image: ...
    def has_platform(self, platform): ...
    def reload(self) -> None: ...

class ImageCollection(Collection[Image]):
    model: type[Image]
    def build(
        self,
        *,
        path: str | None = None,
        fileobj: StringIO | IO[bytes] | None = None,
        tag: str | None = None,
        quiet: bool = False,
        nocache: bool = False,
        rm: bool = False,
        timeout: int | None = None,
        custom_context: bool = False,
        encoding: str | None = None,
        pull: bool = False,
        forcerm: bool = False,
        dockerfile: str | None = None,
        buildargs: dict[str, Any] | None = None,
        container_limits: _ContainerLimits | None = None,
        shmsize: int | None = None,
        labels: dict[str, Any] | None = None,
        # need to use list, because the type must be json serializable
        cache_from: list[str] | None = None,
        target: str | None = None,
        network_mode: str | None = None,
        squash: bool | None = None,
        extra_hosts: list[str] | dict[str, str] | None = None,
        platform: str | None = None,
        isolation: str | None = None,
        use_config_proxy: bool = True,
    ) -> tuple[Image, Iterator[JSON]]: ...
    def get(self, name: str) -> Image: ...
    def get_registry_data(self, name, auth_config: dict[str, Any] | None = None) -> RegistryData: ...
    def list(self, name: str | None = None, all: bool = False, filters: dict[str, Any] | None = None) -> _ImageList: ...
    def load(self, data: bytes | SupportsRead[bytes]) -> _ImageList: ...
    @overload
    def pull(
        self,
        repository: str,
        tag: str | None = None,
        all_tags: Literal[False] = False,
        *,
        platform: str | None = None,
        auth_config: dict[str, Any] | None = None,
    ) -> Image: ...
    @overload
    def pull(
        self,
        repository: str,
        tag: str | None = None,
        *,
        all_tags: Literal[True],
        auth_config: dict[str, Any] | None = None,
        platform: str | None = None,
    ) -> _ImageList: ...
    @overload
    def pull(
        self,
        repository: str,
        tag: str | None,
        all_tags: Literal[True],
        *,
        auth_config: dict[str, Any] | None = None,
        platform: str | None = None,
    ) -> _ImageList: ...
    def push(self, repository: str, tag: str | None = None, **kwargs): ...
    def remove(self, *args, **kwargs) -> None: ...
    def search(self, *args, **kwargs): ...
    def prune(self, filters: dict[str, Any] | None = None): ...
    def prune_builds(self, *args, **kwargs): ...

def normalize_platform(platform, engine_info): ...



from collections.abc import Callable
from typing import IO, Any
from typing_extensions import TypeAlias

from ..main import ModulesType, QRCode
from .styles.moduledrawers.base import QRModuleDrawer

# The second element of the value tuple are keyword arguments used when
# constructing instances of the QRModuleDrawer type.
DrawerAliases: TypeAlias = dict[str, tuple[type[QRModuleDrawer], dict[str, Any]]]

class BaseImage:
    kind: str | None
    allowed_kinds: tuple[str] | None
    needs_context: bool
    needs_processing: bool
    needs_drawrect: bool
    border: int
    width: int
    box_size: int
    pixel_size: int
    modules: list[list[bool | None]]
    # the class accepts arbitrary additional positional arguments to accommodate
    # subclasses with additional arguments. kwargs are forwarded to the `new_image()` call.
    def __init__(
        self, border: int, width: int, box_size: int, *args: Any, qrcode_modules: ModulesType | None, **kwargs: Any
    ) -> None: ...
    def drawrect(self, row: int, col: int) -> None: ...
    def drawrect_context(self, row: int, col: int, qr: QRCode[Any]) -> None: ...
    def process(self) -> None: ...
    def save(self, stream: IO[bytes], kind: str | None = None) -> None: ...
    def pixel_box(self, row: int, col: int) -> tuple[tuple[int, int], tuple[int, int]]: ...
    # the new_image method accepts arbitrary keyword arguments to accommodate
    # subclasses with additional arguments.
    def new_image(self, **kwargs: Any) -> Any: ...
    def init_new_image(self) -> None: ...
    # the get_image method accepts arbitrary keyword arguments to accommodate
    # subclasses with additional arguments.
    def get_image(self, **kwargs: Any) -> Any: ...
    def check_kind(self, kind: str | None, transform: Callable[[str | None], str | None] | None = None) -> str | None: ...
    def is_eye(self, row: int, col: int) -> bool: ...

class BaseImageWithDrawer(BaseImage):
    default_drawer_class: type[QRModuleDrawer]
    drawer_aliases: DrawerAliases
    def get_default_module_drawer(self) -> QRModuleDrawer: ...
    def get_default_eye_drawer(self) -> QRModuleDrawer: ...
    needs_context: bool
    module_drawer: QRModuleDrawer
    eye_drawer: QRModuleDrawer
    # the class accepts arbitrary additional positional arguments to accommodate
    # subclasses with additional arguments. kwargs are forwarded to the `new_image()` call
    # via the BaseImage.__init__ method.
    def __init__(
        self,
        border: int,
        width: int,
        box_size: int,
        *args: Any,
        qrcode_modules: ModulesType | None,
        module_drawer: QRModuleDrawer | str | None = None,
        eye_drawer: QRModuleDrawer | str | None = None,
        **kwargs: Any,
    ) -> None: ...
    def get_drawer(self, drawer: QRModuleDrawer | str | None) -> QRModuleDrawer | None: ...
    def init_new_image(self) -> None: ...
    def drawrect_context(self, row: int, col: int, qr: QRCode[Any]) -> None: ...



from enum import Enum
from typing import Final

class _Days(Enum):
    ROSH_HASHANA = "Rosh Hashana"
    YOM_KIPPUR = "Yom Kippur"
    SUCCOS = "Succos"
    SHMINI_ATZERES = "Shmini Atzeres"
    SIMCHAS_TORAH = "Simchas Torah"
    CHANUKA = "Chanuka"
    TU_BSHVAT = "Tu B'shvat"
    PURIM_KATAN = "Purim Katan"
    PURIM = "Purim"
    SHUSHAN_PURIM = "Shushan Purim"
    PESACH = "Pesach"
    PESACH_SHENI = "Pesach Sheni"
    LAG_BAOMER = "Lag Ba'omer"
    SHAVUOS = "Shavuos"
    TU_BAV = "Tu B'av"
    TZOM_GEDALIA = "Tzom Gedalia"
    TENTH_OF_TEVES = "10 of Teves"
    TAANIS_ESTHER = "Taanis Esther"
    SEVENTEENTH_OF_TAMUZ = "17 of Tamuz"
    NINTH_OF_AV = "9 of Av"

MONTH_NAMES: Final[list[str]]
MONTH_NAMES_HEBREW: Final[list[str]]
FAST_DAYS: Final[list[str]]
FAST_DAYS_HEBREW: Final[list[str]]
FESTIVALS: Final[list[str]]
FESTIVALS_HEBREW: Final[list[str]]
WEEKDAYS: Final[dict[int, str]]



import typing_extensions
from typing import TypedDict

from tensorflow.config import PhysicalDevice

class _MemoryInfo(TypedDict):
    current: int
    peak: int

def get_memory_info(device: str) -> _MemoryInfo: ...
def reset_memory_stats(device: str) -> None: ...
@typing_extensions.deprecated("This function is deprecated in favor of tf.config.experimental.get_memory_info")
def get_memory_usage(device: PhysicalDevice) -> int: ...
def get_memory_growth(device: PhysicalDevice) -> bool: ...
def set_memory_growth(device: PhysicalDevice, enable: bool) -> None: ...
def __getattr__(name: str): ...  # incomplete module



from _typeshed import Incomplete
from collections.abc import Callable, Generator

from networkx.algorithms.flow import edmonds_karp
from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["edge_disjoint_paths", "node_disjoint_paths"]
default_flow_func = edmonds_karp

@_dispatchable
def edge_disjoint_paths(
    G: Graph[_Node],
    s: _Node,
    t: _Node,
    flow_func: Callable[..., Incomplete] | None = None,
    cutoff: int | None = None,
    auxiliary: DiGraph[_Node] | None = None,
    residual: DiGraph[_Node] | None = None,
) -> Generator[Incomplete, None, None]: ...
@_dispatchable
def node_disjoint_paths(
    G: Graph[_Node],
    s: _Node,
    t: _Node,
    flow_func: Callable[..., Incomplete] | None = None,
    cutoff: int | None = None,
    auxiliary: DiGraph[_Node] | None = None,
    residual: DiGraph[_Node] | None = None,
) -> Generator[Incomplete, None, None]: ...



from _typeshed import Incomplete
from collections.abc import Callable, Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["clustering", "average_clustering", "latapy_clustering", "robins_alexander_clustering"]

def cc_dot(nu, nv) -> float: ...
def cc_max(nu, nv) -> float: ...
def cc_min(nu, nv) -> float: ...

modes: dict[str, Callable[[Incomplete, Incomplete], float]]

@_dispatchable
def latapy_clustering(
    G: Graph[_Node], nodes: Iterable[Incomplete] | None = None, mode: str = "dot"
) -> dict[Incomplete, Incomplete]: ...

clustering = latapy_clustering

@_dispatchable
def average_clustering(G: Graph[_Node], nodes: Iterable[Incomplete] | None = None, mode: str = "dot") -> float: ...
@_dispatchable
def robins_alexander_clustering(G: Graph[_Node]) -> float: ...



from _typeshed import Incomplete

from .base import AuthenticationBase

class Database(AuthenticationBase):
    def signup(
        self,
        email: str,
        password: str,
        connection: str,
        username: str | None = None,
        user_metadata: dict[str, Incomplete] | None = None,
        given_name: str | None = None,
        family_name: str | None = None,
        name: str | None = None,
        nickname: str | None = None,
        picture: str | None = None,
    ) -> dict[str, Incomplete]: ...
    def change_password(
        self, email: str, connection: str, password: str | None = None, organization: str | None = None
    ) -> str: ...



import datetime

from ..core import WesternCalendar

class Colombia(WesternCalendar):
    def get_epiphany(self, year: int) -> datetime.date: ...
    def get_saint_joseph(self, year: int) -> datetime.date: ...
    def get_ascension(self, year: int) -> datetime.date: ...
    def get_sacred_heart(self, year: int) -> datetime.date: ...
    def get_saint_peter_and_saint_paul(self, year: int) -> datetime.date: ...
    def get_assumption(self, year: int) -> datetime.date: ...
    def get_day_of_the_races(self, year: int) -> datetime.date: ...
    def get_all_saints(self, year: int) -> datetime.date: ...
    def get_cartagena_independence(self, year: int) -> datetime.date: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.attr_value_pb2
import tensorflow.core.framework.full_type_pb2
import tensorflow.core.framework.resource_handle_pb2
import tensorflow.core.framework.types_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class OpDef(google.protobuf.message.Message):
    """Defines an operation. A NodeDef in a GraphDef specifies an Op by
    using the "op" field which should match the name of a OpDef.
    LINT.IfChange
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class ArgDef(google.protobuf.message.Message):
        """For describing inputs and outputs."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        DESCRIPTION_FIELD_NUMBER: builtins.int
        TYPE_FIELD_NUMBER: builtins.int
        TYPE_ATTR_FIELD_NUMBER: builtins.int
        NUMBER_ATTR_FIELD_NUMBER: builtins.int
        TYPE_LIST_ATTR_FIELD_NUMBER: builtins.int
        HANDLE_DATA_FIELD_NUMBER: builtins.int
        IS_REF_FIELD_NUMBER: builtins.int
        EXPERIMENTAL_FULL_TYPE_FIELD_NUMBER: builtins.int
        name: builtins.str
        """Name for the input/output.  Should match the regexp "[a-z][a-z0-9_]*"."""
        description: builtins.str
        """Human readable description."""
        type: tensorflow.core.framework.types_pb2.DataType.ValueType
        """Describes the type of one or more tensors that are accepted/produced
        by this input/output arg.  The only legal combinations are:
        * For a single tensor: either the "type" field is set or the
          "type_attr" field is set to the name of an attr with type "type".
        * For a sequence of tensors with the same type: the "number_attr"
          field will be set to the name of an attr with type "int", and
          either the "type" or "type_attr" field will be set as for
          single tensors.
        * For a sequence of tensors, the "type_list_attr" field will be set
          to the name of an attr with type "list(type)".
        """
        type_attr: builtins.str
        """if specified, attr must have type "type" """
        number_attr: builtins.str
        """if specified, attr must have type "int" """
        type_list_attr: builtins.str
        """If specified, attr must have type "list(type)", and none of
        type, type_attr, and number_attr may be specified.
        """
        is_ref: builtins.bool
        """For inputs: if true, the inputs are required to be refs.
          By default, inputs can be either refs or non-refs.
        For outputs: if true, outputs are refs, otherwise they are not.
        """
        @property
        def handle_data(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
            tensorflow.core.framework.resource_handle_pb2.ResourceHandleProto.DtypeAndShape
        ]:
            """The handle data for resource inputs."""

        @property
        def experimental_full_type(self) -> tensorflow.core.framework.full_type_pb2.FullTypeDef:
            """Experimental. Full type declaration for this argument.
            The full type specification combines type, type_attr, type_list_attr,
            etc. into a unified representation.
            This declaration may contain non-concrete types (for example,
            Tensor<TypeVar<'T'>> is a valid type declaration.

            Note: this is a transient field. The long-term aim is to represent the
            entire OpDef as a single type: a callable. In that context, this field is
            just the type of a single argument.
            """

        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
            description: builtins.str | None = ...,
            type: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
            type_attr: builtins.str | None = ...,
            number_attr: builtins.str | None = ...,
            type_list_attr: builtins.str | None = ...,
            handle_data: (
                collections.abc.Iterable[tensorflow.core.framework.resource_handle_pb2.ResourceHandleProto.DtypeAndShape] | None
            ) = ...,
            is_ref: builtins.bool | None = ...,
            experimental_full_type: tensorflow.core.framework.full_type_pb2.FullTypeDef | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["experimental_full_type", b"experimental_full_type"]) -> builtins.bool: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "description",
                b"description",
                "experimental_full_type",
                b"experimental_full_type",
                "handle_data",
                b"handle_data",
                "is_ref",
                b"is_ref",
                "name",
                b"name",
                "number_attr",
                b"number_attr",
                "type",
                b"type",
                "type_attr",
                b"type_attr",
                "type_list_attr",
                b"type_list_attr",
            ],
        ) -> None: ...

    @typing.final
    class AttrDef(google.protobuf.message.Message):
        """Description of the graph-construction-time configuration of this
        Op.  That is to say, this describes the attr fields that will
        be specified in the NodeDef.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        TYPE_FIELD_NUMBER: builtins.int
        DEFAULT_VALUE_FIELD_NUMBER: builtins.int
        DESCRIPTION_FIELD_NUMBER: builtins.int
        HAS_MINIMUM_FIELD_NUMBER: builtins.int
        MINIMUM_FIELD_NUMBER: builtins.int
        ALLOWED_VALUES_FIELD_NUMBER: builtins.int
        name: builtins.str
        """A descriptive name for the argument.  May be used, e.g. by the
        Python client, as a keyword argument name, and so should match
        the regexp "[a-z][a-z0-9_]+".
        """
        type: builtins.str
        """One of the type names from attr_value.proto ("string", "list(string)",
        "int", etc.).
        """
        description: builtins.str
        """Human-readable description."""
        has_minimum: builtins.bool
        """--- Constraints ---
        These constraints are only in effect if specified.  Default is no
        constraints.

        For type == "int", this is a minimum value.  For "list(___)"
        types, this is the minimum length.
        """
        minimum: builtins.int
        @property
        def default_value(self) -> tensorflow.core.framework.attr_value_pb2.AttrValue:
            """A reasonable default for this attribute if the user does not supply
            a value.  If not specified, the user must supply a value.
            """

        @property
        def allowed_values(self) -> tensorflow.core.framework.attr_value_pb2.AttrValue:
            """The set of allowed values.  Has type that is the "list" version
            of the "type" field above (uses the "list" field of AttrValue).
            If type == "type" or "list(type)" above, then the "type" field
            of "allowed_values.list" has the set of allowed DataTypes.
            If type == "string" or "list(string)", then the "s" field of
            "allowed_values.list" has the set of allowed strings.
            """

        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
            type: builtins.str | None = ...,
            default_value: tensorflow.core.framework.attr_value_pb2.AttrValue | None = ...,
            description: builtins.str | None = ...,
            has_minimum: builtins.bool | None = ...,
            minimum: builtins.int | None = ...,
            allowed_values: tensorflow.core.framework.attr_value_pb2.AttrValue | None = ...,
        ) -> None: ...
        def HasField(
            self, field_name: typing.Literal["allowed_values", b"allowed_values", "default_value", b"default_value"]
        ) -> builtins.bool: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "allowed_values",
                b"allowed_values",
                "default_value",
                b"default_value",
                "description",
                b"description",
                "has_minimum",
                b"has_minimum",
                "minimum",
                b"minimum",
                "name",
                b"name",
                "type",
                b"type",
            ],
        ) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    INPUT_ARG_FIELD_NUMBER: builtins.int
    OUTPUT_ARG_FIELD_NUMBER: builtins.int
    CONTROL_OUTPUT_FIELD_NUMBER: builtins.int
    ATTR_FIELD_NUMBER: builtins.int
    DEPRECATION_FIELD_NUMBER: builtins.int
    SUMMARY_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    IS_COMMUTATIVE_FIELD_NUMBER: builtins.int
    IS_AGGREGATE_FIELD_NUMBER: builtins.int
    IS_STATEFUL_FIELD_NUMBER: builtins.int
    ALLOWS_UNINITIALIZED_INPUT_FIELD_NUMBER: builtins.int
    IS_DISTRIBUTED_COMMUNICATION_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Op names starting with an underscore are reserved for internal use.
    Names should be CamelCase and match the regexp "[A-Z][a-zA-Z0-9>_]*".
    """
    summary: builtins.str
    """One-line human-readable description of what the Op does."""
    description: builtins.str
    """Additional, longer human-readable description of what the Op does."""
    is_commutative: builtins.bool
    """-------------------------------------------------------------------------
    Which optimizations this operation can participate in.

    True if the operation is commutative ("op(a,b) == op(b,a)" for all inputs)
    """
    is_aggregate: builtins.bool
    """If is_aggregate is true, then this operation accepts N >= 2
    inputs and produces 1 output all of the same type.  Should be
    associative and commutative, and produce output with the same
    shape as the input.  The optimizer may replace an aggregate op
    taking input from multiple devices with a tree of aggregate ops
    that aggregate locally within each device (and possibly within
    groups of nearby devices) before communicating.
    TODO(josh11b): Implement that optimization.
    for things like add
    """
    is_stateful: builtins.bool
    """-------------------------------------------------------------------------
    Optimization constraints.

    Ops are marked as stateful if their behavior depends on some state beyond
    their input tensors (e.g. variable reading op) or if they have
    a side-effect (e.g. printing or asserting ops). Equivalently, stateless ops
    must always produce the same output for the same input and have
    no side-effects.

    By default Ops may be moved between devices.  Stateful ops should
    either not be moved, or should only be moved if that state can also
    be moved (e.g. via some sort of save / restore).
    Stateful ops are guaranteed to never be optimized away by Common
    Subexpression Elimination (CSE).
    for things like variables, queue
    """
    allows_uninitialized_input: builtins.bool
    """-------------------------------------------------------------------------
    Non-standard options.

    By default, all inputs to an Op must be initialized Tensors.  Ops
    that may initialize tensors for the first time should set this
    field to true, to allow the Op to take an uninitialized Tensor as
    input.
    for Assign, etc.
    """
    is_distributed_communication: builtins.bool
    """Indicates whether the op implementation uses distributed communication.
    If True, the op is allowed to return errors for network disconnection and
    trigger TF network failure handling logics.
    """
    @property
    def input_arg(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___OpDef.ArgDef]:
        """Description of the input(s)."""

    @property
    def output_arg(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___OpDef.ArgDef]:
        """Description of the output(s)."""

    @property
    def control_output(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Named control outputs for this operation. Useful only for composite
        operations (i.e. functions) which want to name different control outputs.
        """

    @property
    def attr(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___OpDef.AttrDef]: ...
    @property
    def deprecation(self) -> global___OpDeprecation:
        """Optional deprecation based on GraphDef versions."""

    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        input_arg: collections.abc.Iterable[global___OpDef.ArgDef] | None = ...,
        output_arg: collections.abc.Iterable[global___OpDef.ArgDef] | None = ...,
        control_output: collections.abc.Iterable[builtins.str] | None = ...,
        attr: collections.abc.Iterable[global___OpDef.AttrDef] | None = ...,
        deprecation: global___OpDeprecation | None = ...,
        summary: builtins.str | None = ...,
        description: builtins.str | None = ...,
        is_commutative: builtins.bool | None = ...,
        is_aggregate: builtins.bool | None = ...,
        is_stateful: builtins.bool | None = ...,
        allows_uninitialized_input: builtins.bool | None = ...,
        is_distributed_communication: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["deprecation", b"deprecation"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "allows_uninitialized_input",
            b"allows_uninitialized_input",
            "attr",
            b"attr",
            "control_output",
            b"control_output",
            "deprecation",
            b"deprecation",
            "description",
            b"description",
            "input_arg",
            b"input_arg",
            "is_aggregate",
            b"is_aggregate",
            "is_commutative",
            b"is_commutative",
            "is_distributed_communication",
            b"is_distributed_communication",
            "is_stateful",
            b"is_stateful",
            "name",
            b"name",
            "output_arg",
            b"output_arg",
            "summary",
            b"summary",
        ],
    ) -> None: ...

global___OpDef = OpDef

@typing.final
class OpDeprecation(google.protobuf.message.Message):
    """Information about version-dependent deprecation of an op"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VERSION_FIELD_NUMBER: builtins.int
    EXPLANATION_FIELD_NUMBER: builtins.int
    version: builtins.int
    """First GraphDef version at which the op is disallowed."""
    explanation: builtins.str
    """Explanation of why it was deprecated and what to use instead."""
    def __init__(self, *, version: builtins.int | None = ..., explanation: builtins.str | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["explanation", b"explanation", "version", b"version"]) -> None: ...

global___OpDeprecation = OpDeprecation

@typing.final
class OpList(google.protobuf.message.Message):
    """A collection of OpDefs"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OP_FIELD_NUMBER: builtins.int
    @property
    def op(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___OpDef]: ...
    def __init__(self, *, op: collections.abc.Iterable[global___OpDef] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["op", b"op"]) -> None: ...

global___OpList = OpList



from asgiref.typing import ASGIReceiveCallable, ASGISendCallable
from channels.middleware import BaseMiddleware
from django.contrib.auth.backends import BaseBackend
from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import AnonymousUser
from django.utils.functional import LazyObject

from .consumer import _ChannelScope
from .utils import _ChannelApplication

async def get_user(scope: _ChannelScope) -> AbstractBaseUser | AnonymousUser: ...
async def login(scope: _ChannelScope, user: AbstractBaseUser, backend: BaseBackend | None = None) -> None: ...
async def logout(scope: _ChannelScope) -> None: ...

# Inherits AbstractBaseUser to improve autocomplete and show this is a lazy proxy for a user.
# At runtime, it's just a LazyObject that wraps the actual user instance.
class UserLazyObject(AbstractBaseUser, LazyObject): ...

class AuthMiddleware(BaseMiddleware):
    def populate_scope(self, scope: _ChannelScope) -> None: ...
    async def resolve_scope(self, scope: _ChannelScope) -> None: ...
    async def __call__(
        self, scope: _ChannelScope, receive: ASGIReceiveCallable, send: ASGISendCallable
    ) -> _ChannelApplication: ...

def AuthMiddlewareStack(inner: _ChannelApplication) -> _ChannelApplication: ...



from collections.abc import Callable, Sequence
from typing import Any, Generic, NoReturn, Protocol, TypeVar, overload, type_check_only

from gevent._types import _Loop, _Resolver
from gevent.fileobject import _FileObjectType
from gevent.threadpool import ThreadPool

__all__ = ["config"]

_T = TypeVar("_T")

@type_check_only
class _SettingDescriptor(Protocol[_T]):
    @overload
    def __get__(self, obj: None, owner: type[Config]) -> property: ...
    @overload
    def __get__(self, obj: Config, owner: type[Config]) -> _T: ...
    def __set__(self, obj: Config, value: str | _T) -> None: ...

class SettingType(type):
    def fmt_desc(cls, desc: str) -> str: ...

def validate_invalid(value: object) -> NoReturn: ...
def validate_bool(value: str | bool) -> bool: ...
def validate_anything(value: _T) -> _T: ...

convert_str_value_as_is = validate_anything

class Setting(Generic[_T], metaclass=SettingType):
    order: int  # all subclasses have this
    name: str
    environment_key: str
    value: _T
    default: _T
    document: bool
    desc: str
    validate: Callable[[Any], _T]
    def get(self) -> _T: ...
    def set(self, val: str | _T) -> None: ...

class Config:
    settings: dict[str, Setting[Any]]
    def __init__(self) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: object) -> None: ...
    def set(self, name: str, value: object) -> None: ...
    def __dir__(self) -> list[str]: ...
    def print_help(self) -> None: ...

    # we manually add properties for all the settings in this module
    # SettingType inserts a property into Config for every subclass of Setting
    resolver: _SettingDescriptor[type[_Resolver]]
    threadpool: _SettingDescriptor[type[Threadpool]]
    threadpool_idle_task_timeout: _SettingDescriptor[float]
    loop: _SettingDescriptor[type[_Loop]]
    format_context: _SettingDescriptor[Callable[[Any], str]]
    libev_backend: _SettingDescriptor[str | None]
    fileobject: _SettingDescriptor[_FileObjectType]
    disable_watch_children: _SettingDescriptor[bool]
    track_greenlet_tree: _SettingDescriptor[bool]
    monitor_thread: _SettingDescriptor[bool]
    max_blocking_time: _SettingDescriptor[float]
    memory_monitor_period: _SettingDescriptor[float]
    max_memory_usage: _SettingDescriptor[int | None]
    resolver_nameservers: _SettingDescriptor[Sequence[str] | str | None]
    resolver_timeout: _SettingDescriptor[float | None]
    # these get parsed by gevent.resolver.cares.channel so the Setting does not
    # perform any conversion, but we know at least what types can be valid
    ares_flags: _SettingDescriptor[str | int | None]
    ares_timeout: _SettingDescriptor[str | float | None]
    ares_tries: _SettingDescriptor[str | int | None]
    ares_ndots: _SettingDescriptor[str | int | None]
    ares_udp_port: _SettingDescriptor[str | int | None]
    ares_tcp_port: _SettingDescriptor[str | int | None]
    ares_servers: _SettingDescriptor[Sequence[str] | str | None]
    print_blocking_reports: _SettingDescriptor[bool]

class ImportableSetting(Generic[_T]):
    default: str | Sequence[str]
    shortname_map: dict[str, str]
    def validate(self, value: str | _T) -> _T: ...
    def get_options(self) -> dict[str, _T]: ...

class BoolSettingMixin:
    @staticmethod
    def validate(value: str | bool) -> bool: ...

class IntSettingMixin:
    @staticmethod
    def validate(value: int) -> int: ...

class _PositiveValueMixin(Generic[_T]):
    @staticmethod
    def validate(value: _T) -> _T: ...

class FloatSettingMixin(_PositiveValueMixin[float]): ...
class ByteCountSettingMixin(_PositiveValueMixin[int]): ...

class Resolver(ImportableSetting[type[_Resolver]], Setting[type[_Resolver]]):
    desc: str
    default: list[str]  # type: ignore[assignment]
    shortname_map: dict[str, str]

class Threadpool(ImportableSetting[type[ThreadPool]], Setting[type[ThreadPool]]):
    desc: str
    default: str  # type: ignore[assignment]

class ThreadpoolIdleTaskTimeout(FloatSettingMixin, Setting[float]):
    document: bool
    desc: str
    default: float

class Loop(ImportableSetting[type[_Loop]], Setting[type[_Loop]]):
    desc: str
    default: list[str]  # type: ignore[assignment]
    shortname_map: dict[str, str]

class FormatContext(ImportableSetting[Callable[[Any], str]], Setting[Callable[[Any], str]]):
    default: str  # type: ignore[assignment]

class LibevBackend(Setting[str | None]):
    desc: str
    default: None

class FileObject(ImportableSetting[_FileObjectType], Setting[_FileObjectType]):
    desc: str
    default: list[str]  # type: ignore[assignment]
    shortname_map: dict[str, str]

class WatchChildren(BoolSettingMixin, Setting[bool]):
    desc: str
    default: bool

class TrackGreenletTree(BoolSettingMixin, Setting[bool]):
    default: bool
    desc: str

class MonitorThread(BoolSettingMixin, Setting[bool]):
    default: bool
    desc: str

class MaxBlockingTime(FloatSettingMixin, Setting[float]):
    default: float
    desc: str

class PrintBlockingReports(BoolSettingMixin, Setting[bool]):
    default: bool
    desc: str

class MonitorMemoryPeriod(FloatSettingMixin, Setting[float]):
    default: int
    desc: str

class MonitorMemoryMaxUsage(ByteCountSettingMixin, Setting[int | None]):
    default: None
    desc: str

class AresSettingMixin:
    document: bool
    @property
    def kwarg_name(self) -> str: ...
    validate: Any  # we just want this to mixin without errors

class AresFlags(AresSettingMixin, Setting[str | int | None]):
    default: None

class AresTimeout(AresSettingMixin, Setting[str | float | None]):
    document: bool
    default: None
    desc: str

class AresTries(AresSettingMixin, Setting[str | int | None]):
    default: None

class AresNdots(AresSettingMixin, Setting[str | int | None]):
    default: None

class AresUDPPort(AresSettingMixin, Setting[str | int | None]):
    default: None

class AresTCPPort(AresSettingMixin, Setting[str | int | None]):
    default: None

class AresServers(AresSettingMixin, Setting[Sequence[str] | str | None]):
    document: bool
    default: None
    desc: str

class ResolverNameservers(AresSettingMixin, Setting[Sequence[str] | str | None]):
    document: bool
    default: None
    desc: str
    @property
    def kwarg_name(self) -> str: ...

class ResolverTimeout(FloatSettingMixin, AresSettingMixin, Setting[float | None]):
    document: bool
    desc: str
    @property
    def kwarg_name(self) -> str: ...

config: Config = ...



from collections.abc import Callable, Iterator, Mapping
from typing import TypeVar
from typing_extensions import Self

_T = TypeVar("_T")
_U = TypeVar("_U")
_V = TypeVar("_V")

__all__ = [
    "AtlasView",
    "AdjacencyView",
    "MultiAdjacencyView",
    "UnionAtlas",
    "UnionAdjacency",
    "UnionMultiInner",
    "UnionMultiAdjacency",
    "FilterAtlas",
    "FilterAdjacency",
    "FilterMultiInner",
    "FilterMultiAdjacency",
]

class AtlasView(Mapping[_T, dict[_U, _V]]):
    __slots__ = ("_atlas",)
    def __getstate__(self) -> dict[str, Mapping[_T, dict[_U, _V]]]: ...
    def __setstate__(self, state: dict[str, Mapping[_T, dict[_U, _V]]]) -> None: ...
    def __init__(self, d: Mapping[_T, dict[_U, _V]]) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_T]: ...
    def __getitem__(self, key: _T) -> dict[_U, _V]: ...
    def copy(self) -> dict[_T, dict[_U, _V]]: ...

class AdjacencyView(AtlasView[_T, _U, _V]):
    __slots__ = ()

class MultiAdjacencyView(AdjacencyView[_T, _U, _V]):
    __slots__ = ()

class UnionAtlas(Mapping[_T, dict[_U, _V]]):
    __slots__ = ("_succ", "_pred")
    def __init__(self, succ: AtlasView[_T, _U, _V], pred: AtlasView[_T, _U, _V]) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_T]: ...
    def __getitem__(self, key: _T) -> dict[_U, _V]: ...
    def copy(self) -> Self: ...

class UnionAdjacency(Mapping[_T, dict[_U, _V]]):
    __slots__ = ("_succ", "_pred")
    def __init__(self, succ: AdjacencyView[_T, _U, _V], pred: AdjacencyView[_T, _U, _V]) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_T]: ...
    def __getitem__(self, key: _T) -> dict[_U, _V]: ...
    def copy(self) -> Self: ...

class UnionMultiInner(UnionAtlas[_T, _U, _V]):
    __slots__ = ()

class UnionMultiAdjacency(UnionAdjacency[_T, _U, _V]):
    __slots__ = ()

class FilterAtlas(Mapping[_T, _U]):
    NODE_OK: Callable[[_T], bool]
    def __init__(self, d: Mapping[_T, _U], NODE_OK: Callable[[_T], bool]) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_T]: ...
    def __getitem__(self, key: _T) -> _U: ...

class FilterAdjacency(Mapping[_T, Mapping[_U, _V]]):
    NODE_OK: Callable[[_T], bool]
    EDGE_OK: Callable[[_T, _T], bool]
    def __init__(
        self, d: Mapping[_T, Mapping[_U, _V]], NODE_OK: Callable[[_T], bool], EDGE_OK: Callable[[_T, _T], bool]
    ) -> None: ...
    def __len__(self): ...
    def __iter__(self): ...
    def __getitem__(self, node: _T) -> FilterAtlas[_U, _V]: ...

class FilterMultiInner(FilterAdjacency[_T, _U, _V]): ...
class FilterMultiAdjacency(FilterAdjacency[_T, _U, _V]): ...



from _typeshed import Incomplete
from collections.abc import Iterator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["asyn_fluidc"]

@_dispatchable
def asyn_fluidc(G: Graph[_Node], k: int, max_iter: int = 100, seed: int | RandomState | None = None) -> Iterator[Incomplete]: ...



from collections.abc import Iterator
from typing import NamedTuple, SupportsIndex

from ._types import ErrorCorrect

EXP_TABLE: list[int]
LOG_TABLE: list[int]
RS_BLOCK_OFFSET: dict[ErrorCorrect, int]
RS_BLOCK_TABLE: tuple[tuple[int, int, int] | tuple[int, int, int, int, int, int], ...]

def glog(n: int) -> int: ...
def gexp(n: int) -> int: ...

class Polynomial:
    num: list[int]
    def __init__(self, num: list[int], shift: int) -> None: ...
    def __getitem__(self, index: SupportsIndex) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    def __len__(self) -> int: ...
    def __mul__(self, other: Polynomial) -> Polynomial: ...
    def __mod__(self, other: Polynomial) -> Polynomial: ...

class RSBlock(NamedTuple):
    total_count: int
    data_count: int

def rs_blocks(version: int, error_correction: ErrorCorrect) -> list[RSBlock]: ...



from _typeshed import SupportsWrite

from .messages import Message

class Reporter:
    def __init__(self, warningStream: SupportsWrite[str], errorStream: SupportsWrite[str]) -> None: ...
    def unexpectedError(self, filename: str, msg: str) -> None: ...
    def syntaxError(self, filename: str, msg: str, lineno: int, offset: int | None, text: str | None) -> None: ...
    def flake(self, message: Message) -> None: ...



from binascii import Incomplete
from typing import Final

import grpc

GRPC_GENERATED_VERSION: Final[str]
GRPC_VERSION: Final[str]

class ServerReflectionStub:
    ServerReflectionInfo: Incomplete
    def __init__(self, channel: grpc.Channel) -> None: ...

class ServerReflectionServicer:
    def ServerReflectionInfo(self, request_iterator, context): ...

def add_ServerReflectionServicer_to_server(servicer, server): ...

class ServerReflection:
    @staticmethod
    def ServerReflectionInfo(
        request_iterator,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ): ...



from typing import Any, Literal, overload

from matplotlib.axes import Axes  # type: ignore[import-not-found]
from matplotlib.lines import Line2D  # type: ignore[import-not-found]
from matplotlib.patches import PathPatch  # type: ignore[import-not-found]
from matplotlib.typing import ColorType  # type: ignore[import-not-found]

from .geometry import LinearRing, LineString, MultiLineString, MultiPolygon, Polygon
from .lib import Geometry

def patch_from_polygon(polygon: Polygon | MultiPolygon, **kwargs: Any) -> PathPatch: ...
@overload
def plot_polygon(
    polygon: Polygon | MultiPolygon,
    ax: Axes | None = None,
    add_points: Literal[True] = True,
    color: ColorType | None = None,
    facecolor: ColorType | None = None,
    edgecolor: ColorType | None = None,
    linewidth: float | None = None,
    **kwargs: Any,
) -> tuple[PathPatch, Line2D]: ...
@overload
def plot_polygon(
    polygon: Polygon | MultiPolygon,
    ax: Axes | None = None,
    *,
    add_points: Literal[False],
    color: ColorType | None = None,
    facecolor: ColorType | None = None,
    edgecolor: ColorType | None = None,
    linewidth: float | None = None,
    **kwargs: Any,
) -> PathPatch: ...
@overload
def plot_polygon(
    polygon: Polygon | MultiPolygon,
    ax: Axes | None,
    add_points: Literal[False],
    color: ColorType | None = None,
    facecolor: ColorType | None = None,
    edgecolor: ColorType | None = None,
    linewidth: float | None = None,
    **kwargs: Any,
) -> PathPatch: ...
@overload
def plot_line(
    line: LineString | LinearRing | MultiLineString,
    ax: Axes | None = None,
    add_points: Literal[True] = True,
    color: ColorType | None = None,
    linewidth: float = 2,
    **kwargs: Any,
) -> tuple[PathPatch, Line2D]: ...
@overload
def plot_line(
    line: LineString | LinearRing | MultiLineString,
    ax: Axes | None = None,
    *,
    add_points: Literal[False],
    color: ColorType | None = None,
    linewidth: float = 2,
    **kwargs: Any,
) -> PathPatch: ...
@overload
def plot_line(
    line: LineString | LinearRing | MultiLineString,
    ax: Axes | None,
    add_points: Literal[False],
    color: ColorType | None = None,
    linewidth: float = 2,
    **kwargs: Any,
) -> PathPatch: ...
def plot_points(
    geom: Geometry, ax: Axes | None = None, color: ColorType | None = None, marker: str = "o", **kwargs: Any
) -> Line2D: ...



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import TypedDict, type_check_only

from braintree.configuration import Configuration
from braintree.environment import Environment
from braintree.util.http import Http

@type_check_only
class _Extension(TypedDict):
    errorClass: Incomplete
    legacyCode: int | None

@type_check_only
class _Error(TypedDict):
    attribute: str | None
    code: int | None
    message: str | None
    extensions: _Extension | None

@type_check_only
class _ValidationErrors(TypedDict):
    errors: Iterable[_Error]

@type_check_only
class _Response(TypedDict):
    errors: Iterable[_Error] | None

class GraphQLClient(Http):
    @staticmethod
    def raise_exception_for_graphql_error(response: _Response) -> None: ...
    graphql_headers: dict[str, str]
    def __init__(self, config: Configuration | None = None, environment: Environment | None = None) -> None: ...
    def query(self, definition, variables=None, operation_name=None): ...
    @staticmethod
    def get_validation_errors(response) -> _ValidationErrors | None: ...
    @staticmethod
    def get_validation_error_code(error: _Error) -> int | None: ...



import logging
import socket
from collections.abc import Mapping
from datetime import timedelta

from gunicorn.config import Config
from gunicorn.glogging import Logger
from gunicorn.http import Request
from gunicorn.http.wsgi import Response

from .._types import _EnvironType
from ..glogging import _LogLevelType

METRIC_VAR: str
VALUE_VAR: str
MTYPE_VAR: str
GAUGE_TYPE: str
COUNTER_TYPE: str
HISTOGRAM_TYPE: str

class Statsd(Logger):
    prefix: str
    sock: socket.socket | None
    dogstatsd_tags: str | None
    cfg: Config

    def __init__(self, cfg: Config) -> None: ...
    def critical(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def error(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def warning(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def info(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def debug(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def exception(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = True,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def log(
        self,
        lvl: _LogLevelType,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def access(self, resp: Response, req: Request, environ: _EnvironType, request_time: timedelta) -> None: ...
    def gauge(self, name: str, value: float) -> None: ...
    def increment(self, name: str, value: int, sampling_rate: float = 1.0) -> None: ...
    def decrement(self, name: str, value: int, sampling_rate: float = 1.0) -> None: ...
    def histogram(self, name: str, value: float) -> None: ...



from typing import Any
from typing_extensions import Self

from requests import Response, Session
from requests.sessions import _Auth, _Data, _Files, _ParamsMappingValueType

from .serialize import Serializer

__all__ = ["Resource", "API"]

class ResourceAttributesMixin:
    # Exists at runtime:
    def __getattr__(self, item: str) -> Any: ...

class Resource(ResourceAttributesMixin):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __call__(self, id: str | None = None, format: str | None = None, url_override: str | None = None) -> Self: ...
    def get(self, **kwargs: _ParamsMappingValueType) -> Response: ...
    def options(self, **kwargs: _ParamsMappingValueType) -> Response: ...
    def head(self, **kwargs: _ParamsMappingValueType) -> Response: ...
    def post(self, data: _Data | None = None, files: _Files | None = None, **kwargs: _ParamsMappingValueType) -> Response: ...
    def patch(self, data: _Data | None = None, files: _Files | None = None, **kwargs: _ParamsMappingValueType) -> Response: ...
    def put(self, data: _Data | None = None, files: _Files | None = None, **kwargs: _ParamsMappingValueType) -> Response: ...
    def delete(self, **kwargs: _ParamsMappingValueType) -> Response: ...
    def url(self) -> str: ...

class API(ResourceAttributesMixin):
    resource_class: type[Resource]
    def __init__(
        self,
        base_url: str | None = None,
        auth: _Auth | None = None,
        format: str | None = None,
        append_slash: bool = True,
        session: Session | None = None,
        serializer: Serializer | None = None,
    ) -> None: ...



import datetime

from ..core import WesternCalendar

class UnitedKingdom(WesternCalendar):
    non_computable_holiday_dict: dict[int, list[tuple[datetime.date, str]]]
    def get_early_may_bank_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_spring_bank_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_late_summer_bank_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def non_computable_holiday(self, year: int) -> tuple[datetime.date, str] | None: ...

class UnitedKingdomNorthernIreland(UnitedKingdom): ...



from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Ldap(VaultApiBase):
    def configure(
        self,
        userdn=None,
        groupdn=None,
        url=None,
        case_sensitive_names=None,
        starttls=None,
        tls_min_version=None,
        tls_max_version=None,
        insecure_tls=None,
        certificate=None,
        binddn=None,
        bindpass=None,
        userattr=None,
        discoverdn=None,
        deny_null_bind: bool = True,
        upndomain=None,
        groupfilter=None,
        groupattr=None,
        use_token_groups=None,
        token_ttl=None,
        token_max_ttl=None,
        mount_point="ldap",
        *,
        anonymous_group_search=None,
        client_tls_cert=None,
        client_tls_key=None,
        connection_timeout=None,
        dereference_aliases=None,
        max_page_size=None,
        request_timeout=None,
        token_bound_cidrs=None,
        token_explicit_max_ttl=None,
        token_no_default_policy=None,
        token_num_uses=None,
        token_period=None,
        token_policies=None,
        token_type=None,
        userfilter=None,
        username_as_alias=None,
    ): ...
    def read_configuration(self, mount_point="ldap"): ...
    def create_or_update_group(self, name, policies=None, mount_point="ldap"): ...
    def list_groups(self, mount_point="ldap"): ...
    def read_group(self, name, mount_point="ldap"): ...
    def delete_group(self, name, mount_point="ldap"): ...
    def create_or_update_user(self, username, policies=None, groups=None, mount_point="ldap"): ...
    def list_users(self, mount_point="ldap"): ...
    def read_user(self, username, mount_point="ldap"): ...
    def delete_user(self, username, mount_point="ldap"): ...
    def login(self, username, password, use_token: bool = True, mount_point="ldap"): ...



from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Init(SystemBackendMixin):
    def read_init_status(self): ...
    def is_initialized(self): ...
    def initialize(
        self,
        secret_shares=None,
        secret_threshold=None,
        pgp_keys=None,
        root_token_pgp_key=None,
        stored_shares=None,
        recovery_shares=None,
        recovery_threshold=None,
        recovery_pgp_keys=None,
    ): ...



import datetime

from .core import UnitedStates

class Alabama(UnitedStates): ...
class AlabamaBaldwinCounty(Alabama): ...
class AlabamaMobileCounty(Alabama): ...

class AlabamaPerryCounty(Alabama):
    def get_obama_day(self, year: int) -> tuple[datetime.date, str]: ...



import socket
from types import FrameType
from typing import ClassVar

from gunicorn.app.base import BaseApplication
from gunicorn.config import Config
from gunicorn.glogging import Logger as GLogger
from gunicorn.http import Request
from gunicorn.workers.workertmp import WorkerTmp

from .._types import _AddressType, _WSGIAppType
from ..reloader import _ReloaderType

class Worker:
    SIGNALS: ClassVar[list[int]]
    PIPE: ClassVar[list[int]]
    age: int
    pid: str
    ppid: int
    sockets: list[socket.socket]
    app: BaseApplication
    timeout: int
    cfg: Config
    booted: bool
    aborted: bool
    reloader: _ReloaderType | None
    nr: int
    max_requests: int
    alive: bool
    log: GLogger
    tmp: WorkerTmp
    wait_fds: list[socket.socket | int]
    wsgi: _WSGIAppType

    def __init__(
        self, age: int, ppid: int, sockets: list[socket.socket], app: BaseApplication, timeout: int, cfg: Config, log: GLogger
    ) -> None: ...
    def notify(self) -> None: ...
    def run(self) -> None: ...
    def init_process(self) -> None: ...
    def load_wsgi(self) -> None: ...
    def init_signals(self) -> None: ...
    def handle_usr1(self, sig: int, frame: FrameType | None) -> None: ...
    def handle_exit(self, sig: int, frame: FrameType | None) -> None: ...
    def handle_quit(self, sig: int, frame: FrameType | None) -> None: ...
    def handle_abort(self, sig: int, frame: FrameType | None) -> None: ...
    def handle_error(self, req: Request | None, client: socket.socket, addr: _AddressType, exc: BaseException) -> None: ...
    def handle_winch(self, sig: int, fname: str | None) -> None: ...



class ConfigApiMixin:
    def create_config(self, name, data, labels=None, templating=None): ...
    def inspect_config(self, id): ...
    def remove_config(self, id): ...
    def configs(self, filters=None): ...



from _typeshed import Incomplete

from braintree.braintree_gateway import BraintreeGateway
from braintree.util.graphql_client import GraphQLClient
from braintree.util.http import Http

class Configuration:
    @staticmethod
    def configure(
        environment,
        merchant_id: str,
        public_key: str,
        private_key: str,
        *,
        http_strategy=None,
        timeout: int = 60,
        wrap_http_exceptions: bool = False,
    ) -> None: ...
    @staticmethod
    def for_partner(
        environment,
        partner_id: str,
        public_key: str,
        private_key: str,
        *,
        http_strategy=None,
        timeout: int = 60,
        wrap_http_exceptions: bool = False,
    ) -> Configuration: ...
    @staticmethod
    def gateway() -> BraintreeGateway: ...
    @staticmethod
    def instantiate() -> Configuration: ...
    @staticmethod
    def api_version() -> str: ...
    @staticmethod
    def graphql_api_version() -> str: ...
    environment: Incomplete
    merchant_id: str | None
    public_key: str | None
    private_key: str | None
    client_id: str | None
    client_secret: str | None
    access_token: str | None
    timeout: int
    wrap_http_exceptions: bool
    def __init__(
        self,
        environment=None,
        merchant_id: str | None = None,
        public_key: str | None = None,
        private_key: str | None = None,
        client_id: str | None = None,
        client_secret: str | None = None,
        access_token: str | None = None,
        *args,
        timeout: int = 60,
        wrap_http_exceptions: bool = False,
        http_strategy=None,
    ) -> None: ...
    def base_merchant_path(self) -> str: ...
    def base_url(self) -> str: ...
    def graphql_base_url(self) -> str: ...
    def http(self) -> Http: ...
    def graphql_client(self) -> GraphQLClient: ...
    def http_strategy(self): ...
    def has_client_credentials(self) -> bool: ...
    def assert_has_client_credentials(self) -> None: ...
    def has_access_token(self) -> bool: ...



from reportlab.lib.abag import ABag
from reportlab.lib.styles import ParagraphStyle, PropertySet
from reportlab.pdfgen.textobject import PDFTextObject
from reportlab.platypus.flowables import Flowable
from reportlab.platypus.paraparser import ParaFrag

class ParaLines(ABag): ...
class FragLine(ABag): ...

def cleanBlockQuotedText(text: str, joiner: str = " ") -> str: ...

class Paragraph(Flowable):
    text: str
    frags: list[ParaFrag]
    style: ParagraphStyle
    bulletText: str | None
    caseSensitive: int
    encoding: str
    def __init__(
        self,
        text: str,
        # NOTE: This should be a ParagraphStyle
        style: PropertySet | None = None,
        bulletText: str | None = None,
        frags: list[ParaFrag] | None = None,
        caseSensitive: int = 1,
        encoding: str = "utf8",
    ) -> None: ...
    def minWidth(self) -> float: ...
    def draw(self) -> None: ...
    def breakLines(self, width: float | list[float] | tuple[float, ...]) -> ParaLines | ParaFrag: ...
    def breakLinesCJK(self, maxWidths: float | list[float] | tuple[float, ...]) -> ParaLines | ParaFrag: ...
    def beginText(self, x: float, y: float) -> PDFTextObject: ...
    def drawPara(self, debug: int = 0) -> None: ...
    def getPlainText(self, identify: bool | None = None) -> str: ...
    def getActualLineWidths0(self) -> list[float]: ...
    @staticmethod
    def dumpFrags(frags, indent: int = 4, full: bool = False) -> str: ...

__all__ = ("Paragraph", "cleanBlockQuotedText", "ParaLines", "FragLine")



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.charts.areas import PlotArea
from reportlab.graphics.charts.textlabels import Label
from reportlab.graphics.widgetbase import PropHolder
from reportlab.lib.attrmap import *

__version__: Final[str]

class WedgeLabel(Label): ...

class WedgeProperties(PropHolder):
    strokeWidth: int
    fillColor: Incomplete
    strokeColor: Incomplete
    strokeDashArray: Incomplete
    strokeLineJoin: int
    strokeLineCap: int
    strokeMiterLimit: int
    popout: int
    fontName: Incomplete
    fontSize: Incomplete
    fontColor: Incomplete
    labelRadius: float
    label_dx: int
    label_text: Incomplete
    label_topPadding: int
    label_boxAnchor: str
    label_boxStrokeColor: Incomplete
    label_boxStrokeWidth: float
    label_boxFillColor: Incomplete
    label_strokeColor: Incomplete
    label_strokeWidth: float
    label_leading: Incomplete
    label_textAnchor: str
    label_simple_pointer: int
    label_visible: int
    label_pointer_strokeColor: Incomplete
    label_pointer_strokeWidth: float
    label_pointer_elbowLength: int
    label_pointer_edgePad: int
    label_pointer_piePad: int
    visible: int
    shadingKind: Incomplete
    shadingAmount: float
    shadingAngle: float
    shadingDirection: str
    def __init__(self) -> None: ...

class AbstractPieChart(PlotArea):
    def makeSwatchSample(self, rowNo, x, y, width, height): ...
    def getSeriesName(self, i, default=None): ...

def boundsOverlap(P, Q): ...
def findOverlapRun(B, wrap: int = 1): ...
def fixLabelOverlaps(L, sideLabels: bool = False, mult0: float = 1.0) -> None: ...
def intervalIntersection(A, B): ...
def theta0(data, direction): ...

class AngleData(float):
    def __new__(cls, angle, data): ...

class Pie(AbstractPieChart):
    other_threshold: Incomplete
    x: int
    y: int
    width: int
    height: int
    data: Incomplete
    labels: Incomplete
    startAngle: int
    direction: str
    simpleLabels: int
    checkLabelOverlap: int
    pointerLabelMode: Incomplete
    sameRadii: bool
    orderMode: str
    xradius: Incomplete
    sideLabels: int
    sideLabelsOffset: float
    slices: Incomplete
    angleRange: int
    def __init__(self, *, angleRange: int = 360, **kwds) -> None: ...
    def demo(self): ...
    centerx: Incomplete
    centery: Incomplete
    yradius: Incomplete
    lu: Incomplete
    ru: Incomplete
    def makePointerLabels(self, angles, plMode): ...
    def normalizeData(self, keepData: bool = False): ...
    def makeAngles(self): ...
    def makeWedges(self): ...
    def draw(self): ...

class LegendedPie(Pie):
    x: int
    y: int
    height: int
    width: int
    data: Incomplete
    labels: Incomplete
    direction: str
    pieAndLegend_colors: Incomplete
    legendNumberOffset: int
    legendNumberFormat: str
    legend_data: Incomplete
    legend1: Incomplete
    legend_names: Incomplete
    leftPadding: int
    rightPadding: int
    topPadding: int
    bottomPadding: int
    drawLegend: int
    def __init__(self) -> None: ...
    def draw(self): ...
    def demo(self, drawing=None): ...

class Wedge3dProperties(PropHolder):
    strokeWidth: int
    shading: float
    visible: int
    strokeColorShaded: Incomplete
    strokeColor: Incomplete
    strokeDashArray: Incomplete
    popout: int
    fontName: Incomplete
    fontSize: Incomplete
    fontColor: Incomplete
    labelRadius: float
    label_dx: int
    label_text: Incomplete
    label_topPadding: int
    label_boxAnchor: str
    label_boxStrokeColor: Incomplete
    label_boxStrokeWidth: float
    label_boxFillColor: Incomplete
    label_strokeColor: Incomplete
    label_strokeWidth: float
    label_leading: Incomplete
    label_textAnchor: str
    label_visible: int
    label_simple_pointer: int
    def __init__(self) -> None: ...

class _SL3D:
    lo: Incomplete
    hi: Incomplete
    mid: Incomplete
    not360: Incomplete
    def __init__(self, lo, hi) -> None: ...

class Pie3d(Pie):
    perspective: int
    depth_3d: int
    angle_3d: int
    def CX(self, i, d): ...
    def CY(self, i, d): ...
    def OX(self, i, o, d): ...
    def OY(self, i, o, d): ...
    def rad_dist(self, a): ...
    slices: Incomplete
    xradius: Incomplete
    width: int
    height: int
    data: Incomplete
    def __init__(self) -> None: ...
    dy: Incomplete
    def draw(self): ...
    def demo(self): ...

def sample0a(): ...
def sample0b(): ...
def sample1(): ...
def sample2(): ...
def sample3(): ...
def sample4(): ...
def sample5(): ...
def sample6(): ...
def sample7(): ...
def sample8(): ...
def sample9(): ...
